
% Visualize three different families of models for 16 object categories:
% - 16 Independent Dirichlet process mixture models (unshared parts)
% - Hierarchical Dirichlet process mixture model (shared parts)
% - Transformed Dirichlet process mixture model (translated shared parts)

clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DEMO PARAMETERS
ROOTHOME = 'database';   % Folder that has the databases of images
DEMOHOME = 'demoModels'; % Folder that has the trained TDP models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load precomputed image features
load(fullfile(ROOTHOME, 'imagesSharingIJCV', 'sharing_ijcv16.mat'));    

% load models trained from 30 images/category
% DP{1}=Indep. DP bag of feat, DP{2}=HDP bag of feat
% DP{3}=Indep. DP pos of feat, DP{4}=HDP pos of feat
load(fullfile(DEMOHOME, 'objectSharingHDP.mat'));
DP = DP([3 4 2]);
usePos   = [1 1 0];
nummodel = 3;
numobj   = 16;

% plot parameters
partprobThresh = 0.8;
partnumThresh  = 20;
Dx  = 1/4;
Dy  = 1/4;
dim = 2;

% extract classes/tables from learned model(s)
for aa = 1:nummodel
  [classParam,classProb,classTran] = getclasses(DP{aa});
  [tableClass,tableProb,tableTran] = gettables(DP{aa});
  numclass = length(classProb);

  objProb = zeros(numobj,numclass);
  for oo = 1:numobj
    for tt = 1:length(tableProb{oo})
      cc = tableClass{oo}(tt);
      objProb(oo,cc) = objProb(oo,cc) + tableProb{oo}(tt);
    end
  end

  classP{aa} = classParam;
  tableC{aa} = tableClass;
  tableP{aa} = tableProb;
  tableT{aa} = tableTran;
  objP{aa}   = objProb;
end

% images of each object category
for oo = 1:numobj
  objind{oo} = find(imagedata.classes == oo);
end
crntind = ones(numobj,1);

% Create figure
screenSize = get(0, 'ScreenSize'); screenSize(1) = 100; screenSize(2) = 100; 
screenSize(3:4) = screenSize(3:4)-200;
Hfig = figure;
set (Hfig, 'position', screenSize);

while true

% choose category to test next
trueclass = ceil(16*rand(1));

% load test image
objimage = imagedata.images{objind{trueclass}(crntind(trueclass))};
objpos   = imagedata.position{objind{trueclass}(crntind(trueclass))};
testdata{1} = objpos(1:2,:) + 1;
testdata{2} = imagedata.appearance{objind{trueclass}(crntind(trueclass))};
crntind(trueclass) = crntind(trueclass) + 1;

% clear old plots from figure
clf;

for aa = 1:nummodel
  % discard positions for BOF models
  if ~usePos(aa) 
    testdataModel = testdata(2);
  else
    testdataModel = testdata;
  end

  % original image
  Hax = axes('position', [0*Dx (nummodel-aa+0.7)*Dy Dx Dy]);
  image(objimage, 'Parent', Hax); 
  axis(Hax,'equal'); axis(Hax,'tight'); axis(Hax,'off');
  colormap(Hax,gray(256));
  if aa==1 title(Hax,'Test Image'); end

  % posterior probabilities of each object category
  postP = sum(predictlike(DP{aa}, [1:numobj], testdataModel), 2);
  postP = exp((postP - max(postP))/10); % less confident for visualization
  postP = postP/sum(postP);
  [val,algclass] = max(postP);

  Hax = axes('position', [1*Dx (nummodel-aa+0.7)*Dy Dx Dy]);
  bar(Hax, [1:numobj], postP); hold(Hax, 'on');
  bar(Hax, trueclass, postP(trueclass), 'b');
  if algclass ~= trueclass
    bar(Hax, algclass, postP(algclass), 'r');
  end
  axis(Hax, [0 numobj+1 0 1.5]); axis(Hax,'square');
  set(Hax,'XTick',[]); set(Hax,'YTick',[]);
  Htxt = text((numobj+1)/2, 1.3, imagedata.classnames{algclass});
  set(Htxt, 'horizontalalignment', 'center', 'verticalalignment', 'middle');
  set(Htxt, 'fontweight', 'bold');
  if algclass == trueclass
    set(Htxt, 'color', 'b');
  else
    set(Htxt, 'color', 'r');
  end
  if aa==1 
    title(Hax,{'Posterior probability of each category',...
      '(blue=correct, red=incorrect)'}); 
  end

  % find highest probability parts for most likely category
  [partprob,partorder] = sort(-objP{aa}(algclass,:));
  partprob = -partprob;
  lastpart = min(find(cumsum(partprob) > partprobThresh));
  lastpart = min(lastpart, partnumThresh);

  % part assignments for most likely object category 
  [partval,partassign] = max(estassign(DP{aa}, algclass, testdataModel), [], 2);
  plotind = []; plotclass = [];
  for c = lastpart:-1:1
    cc = partorder(c);
    newind    = find(partassign' == cc);
    plotind   = [plotind newind];
    plotclass = [plotclass c*ones(1,length(newind))];
  end

  % plot assignments of features to parts
  partcolor = hsv(lastpart);
  Hax = axes('position', [2*Dx (nummodel-aa+0.7)*Dy Dx Dy]);
  displayROI(objimage, objpos(:,plotind), plotclass, partcolor);
  title(Hax,''); colormap(Hax,gray(256));
  if aa==1 
    title(Hax,{'Features colored according to','most likely corresponding part'}); 
  end

  % plot Gaussian mixture position densities for chosen parts
  Hax = axes('position', [3*Dx (nummodel-aa+0.7)*Dy Dx Dy]);
  if usePos(aa) 
    classMean  = classP{aa}{1}(1:dim,partorder(1:lastpart));
    classCovar = reshape(classP{aa}{1}(dim+1:end, partorder(1:lastpart)), ...
      dim, dim, lastpart);
    classProb  = partprob;

    plotGaussMix(classMean, classCovar, classProb, 'k', Hax, ...
      {'.','MarkerSize',1}, {'-k','LineWidth',3});
    hold(Hax,'on');
    tt = 1;
    for c = 1:lastpart
      cc = partorder(c);
      Hplot = plot(Hax, classMean(1,c), classMean(2,c), 'ok');
      set(Hplot, 'MarkerSize', 8, 'MarkerFaceColor', partcolor(c,:));
    end
    axis(Hax,'square'); axis(Hax,'ij'); axis(Hax,[0 128 0 128]);
    set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);
  else
    axis(Hax,'off');
  end
  if aa==1 
    title(Hax,{'Gaussian position densities for parts','generating 80% of observed features'}); 
  end
end

Hax = axes('position', [0 0 1 1]); axis(Hax,'off');
text(0.2, .15, {'Training was performed on 30 images from each of 16 categories.',...
  'Testing uses parts learned from assignments following 1,000 Gibbs sampling iterations.',...
  '',...
  'Independent DPs (top):  Best accuracy by jointly modeling feature appearance and position.',...
  'Hierarchical DP (middle):  Parts are shared among categories, reducing computation.',...  
  'Bag of features (bottom):  HDP bag of features model shares parts, but ignores positions.',...
  }, 'verticalalignment', 'top');

userIn = input('View another test image (default=yes)? ','s');
if (length(userIn) > 0) && isequal(lower(userIn(1)),'n')
  break;
end
end

