
function posterior = estassign(objpart, group, category)
%estassign    Posterior class assignment probabilities for features 
%
%    classprob = estassign(OP, group, category)
%
%  Posterior probability that each class generated each data point, where
%    group     = single group for which data assignment probabilities computed
%    category  = assumed object category under which to compute probabilities
%                (defaults to category specified when data inserted)
%    post      = (numdata x numclass) matrix of P(class|data,category)

  % default parameters
  if nargin < 3 || category == 0
    category = objpart.category(group);
  end
  if category == 0
    error('Need object category to compute part assignment probabilities');
  end

  if length(group) > 1
    error('Only estimates class assignments for a single group');
  end

  % static parameters
  alpha    = objpart.alpha;
  numclass = objpart.numclass;
  numtype  = objpart.numtype;
  dtype    = objpart.dtype;
  dparam   = objpart.dparam;

  % class (part) level state variables
  classnd = objpart.classnd;
  classqq = objpart.classqq;

  % data for chosen group
  numdata = objpart.numdata(group);
  datass  = objpart.datass{group};

  % compute P(part|object)
  classprob = objpart.objnd(category,:) + alpha;
  classprob = classprob / sum(classprob);

  % compute logP(feature|class)
  clik = zeros(numdata,numclass);
  for dd = 1:numtype
    classqqLocal = classqq{dd};
    datassLocal  = datass{dd};
    dtypeLocal   = dtype(dd);
    dparamLocal  = dparam{dd};

    for ii = 1:numdata
      clik(ii,:) = clik(ii,:) + datastat_condlike(classqqLocal, ...
        classnd, datassLocal(:,ii), dtypeLocal, dparamLocal);
    end
  end

  % compute P(class|feature,object)
  posterior = exp(clik) .* repmat(classprob,numdata,1);
  probnorm  = sum(posterior,2);
  posterior = posterior ./ repmat(probnorm,1,numclass);

