
function objpart = objectpart(numgroup, numobj, numclass, alpha, dataprior)
%objectpart     Models object categories via distributions over shared parts
%
%    OP = objectpart(numgroup, numobj, numclass, alpha, dataprior)
%
%  Construct instance OP of objectpart class, where
%    numgroup  = number of groups of data (images or documents)
%    numobj    = number of categories (objects or authors)
%    numclass  = number of latent classes (parts or topics)
%    alpha     = Dirichlet hyperparameter for category-specific 
%                distributions over classes
%    dataprior = struct of hyperparameters for class-specific feature
%                distributions, or cell array of structs if multiple datatypes
%                (see datastat_init.m for available types and parameters)

  % Default constructor (needed so load has proper functionality)
  if nargin == 0
    numgroup = 1;
    numobj   = 1;
    numclass = 1;
    alpha    = 1;

    dataprior.type       = 'discrete';
    dataprior.numfeature = 1;
    dataprior.dirweight  = 1;
  end
  
  if ~iscell(dataprior)
    dataprior = {dataprior};
  end

  % Dirichlet prior parameters
  objpart.alpha = alpha;

  % Counts: groups, objects, classes/parts 
  objpart.numgroup = numgroup;
  objpart.numobj   = numobj;
  objpart.numclass = numclass;

  % Data items: counts, sufficient statistics, class assignments, categories
  objpart.numtype  = length(dataprior);
  objpart.numdata  = zeros(numgroup,1);
  objpart.datass   = cell(numgroup,1);
  objpart.datacc   = cell(numgroup,1);
  objpart.category = zeros(numgroup,1);

  for jj = 1:numgroup
    objpart.datass{jj} = cell(objpart.numtype,1);
  end

  % Object categories: data item counts
  objpart.objnd    = zeros(numobj,numclass);

  % Classes: data item counts, sufficient statistics, hyperparameters
  objpart.classnd  = zeros(1,numclass);

  for dd = 1:objpart.numtype
    [qq0,dtype,dparam] = datastat_init(dataprior{dd});
    objpart.classqq{dd} = repmat(qq0, 1, numclass);
    objpart.dtype(dd)   = dtype;
    objpart.dparam{dd}  = dparam;
  end

  % Instantiate class
  objpart = class(objpart,'objectpart');

