
function objpart = gibbs_noref(objpart, groups)
%gibbs_noref     Iteration of Gibbs sampler, neglecting reference positions

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if objpart.numtype > 2
  error('Gibbs sampler supports at most two data types');
end
usetypeB = (objpart.numtype > 1);

% static parameters
alpha    = objpart.alpha;
dtype    = objpart.dtype;
dparam   = objpart.dparam{1};

% class (part) level state variables
classnd = objpart.classnd;
classqq = objpart.classqq{1};

% parameters for second datatype
if usetypeB
  dparamB  = objpart.dparam{2};
  classqqB = objpart.classqq{2};
end

% resample groups in random order
for j = randperm(length(groups))
  jj = groups(j);

  % state variables for current group
  numdata  = objpart.numdata(jj);
  datass   = objpart.datass{jj}{1};
  datacc   = objpart.datacc{jj};
  category = objpart.category(jj);
  objnd    = objpart.objnd(category,:);

  % state variables for second datatype
  if usetypeB
    datassB = objpart.datass{jj}{2};
  else
    datassB = zeros(0,numdata);
  end

  % sample part associated with each data item, in random order
  for ii = randperm(numdata)
    ss    = datass(:,ii);
    ssB   = datassB(:,ii);
    oldcc = datacc(ii);

    % temporarily remove data item from model, unless not yet assigned
    if oldcc > 0
      classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
        classnd(oldcc), ss, dtype(1), dparam);
      if usetypeB
        classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
          classnd(oldcc), ssB, dtype(2), dparamB);
      end
      classnd(oldcc)   = classnd(oldcc) - 1;
      objnd(oldcc)     = objnd(oldcc) - 1;
    end

    % compute conditional likelihood factors for each class
    clik = datastat_condlike(classqq, classnd, ss, dtype(1), dparam);
    if usetypeB
      clik = clik + datastat_condlike(classqqB, classnd, ssB, dtype(2), dparamB);
    end

    % sample a new class assignment, accounting for Dirichlet prior
    clik  = exp(clik-max(clik)) .* (objnd + alpha);
    newcc = multinomial_rnd(clik, 1);

    % add data item back to model
    datacc(ii)       = newcc;
    classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
      classnd(newcc), ss, dtype(1), dparam);
    if usetypeB
      classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
        classnd(newcc), ssB, dtype(2), dparamB);
    end
    classnd(newcc)   = classnd(newcc) + 1;
    objnd(newcc)     = objnd(newcc) + 1;
  end

  objpart.datacc{jj}        = datacc;
  objpart.objnd(category,:) = objnd;
end

% store class state variables for next iteration
objpart.classqq{1} = classqq;
objpart.classnd = classnd;
if usetypeB
  objpart.classqq{2} = classqqB;
end

