
function [objpart,probpart,partparam] = ...
  sampledata(objpart, groups, category, numdata, probpart, partparam)
%sampledata     Generate random datasets using specified, or random, parameters
%
%    OP = sampledata(OP,groups,category,numdata,probclass,classparam)
%
%  Generate synthetic dataset by random sampling, where
%    groups     = groups for which synthetic data created (all, by default)
%    category   = object category for specified groups, or 0 to sample
%                 uniformly among categories for each group (default)
%    numdata    = number of new data items for each group
%    probclass  = (numobj x numclass) matrix of P(class|category), or empty
%                 to sample this randomly from Dirichlet prior
%    classparam = parameters for each class, or empty to randomly sample
%                 using densities implied by data hyperparameters


  % Default parameters
  if nargin < 2
    groups = [];
  end
  if nargin < 3
    category = 0;
  end
  if nargin < 4
    numdata = 100;
  end
  if nargin < 5
    probpart = [];
  end
  if nargin < 6
    partparam = {};
  end

  if ~iscell(partparam)
    partparam = {partparam};
  end

  if isempty(groups)
    groups = [1:objpart.numgroup];
  end

  % Hyperparameters
  numobj   = objpart.numobj;
  numtype  = objpart.numtype;
  numclass = objpart.numclass;
  alpha    = objpart.alpha;
  dtype    = objpart.dtype;
  dparam   = objpart.dparam;

  % Class (part) state variables
  classqq = objpart.classqq;
  classnd = objpart.classnd;
  objnd   = objpart.objnd;

  % Determine category labels and p(part|category)
  if category == 0
    datacat = multinomial_rnd(ones(numobj,1), length(groups));
  else
    datacat = category*ones(length(groups),1);
  end

  if isempty(probpart)
    probpart = dirichlet_rnd(alpha*ones(numclass,1), numobj)';
  elseif size(probpart,1) == 1
    probpart = repmat(probpart,numobj,1);
  end

  % Determine parameters associated with each part
  if isempty(partparam)
    for dd = 1:numtype
      for cc = 1:numclass
        partparam{dd}(:,cc) = datastat_sampparam(classqq{dd}(:,cc), ...
          classnd(cc), 1, dtype(dd), dparam{dd});
      end
    end
  else
    for dd = 1:numtype 
      if size(partparam{dd},2) ~= numclass
        error('Improper number of part parameters');
      end
    end
  end

  % Sample dataset for each specified group
  for j = 1:length(groups)
    jj = groups(j);

    % Check consistency of category label
    if (objpart.category(jj) > 0) && (datacat(j) ~= objpart.category(jj))
      error('Supplied category label does not match this group');
    end
    objpart.category(jj) = datacat(j);
    
    % Independently sample each data item
    newdatass = cell(numtype,1);
    newdatacc = zeros(1,numdata);
    for ii = 1:numdata
      % sample part associated with data item
      newdatacc(ii) = multinomial_rnd(probpart(datacat(j),:), 1);

      % sample data item using part parameters, and update part statistics
      for dd = 1:numtype
        newdatass{dd}(:,ii) = dataparam_sampdata(partparam{dd}(:,newdatacc(ii)), ...
          1, dtype(dd), dparam{dd});
        classqq{dd}(:,newdatacc(ii)) = datastat_add(classqq{dd}(:,newdatacc(ii)), ...
          classnd(newdatacc(ii)), newdatass{dd}(:,ii), dtype(dd), dparam{dd});
      end

      classnd(newdatacc(ii)) = classnd(newdatacc(ii)) + 1;
      objnd(datacat(j),newdatacc(ii)) = objnd(datacat(j),newdatacc(ii)) + 1;
    end

    % Store data and generating class assignments
    objpart.numdata(jj)  = objpart.numdata(jj) + numdata;
    objpart.datacc{jj}   = [objpart.datacc{jj}, newdatacc];
    for dd = 1:numtype
      objpart.datass{jj}{dd} = [objpart.datass{jj}{dd}, newdatass{dd}];
    end
  end

  objpart.classqq = classqq;
  objpart.classnd = classnd;
  objpart.objnd   = objnd;

