
function sceneobj = clearlabels(sceneobj, groups, clearcat, sampref)
%clearlabels     Clear assignments of data to parts/objects for specified groups
%
%     SO = clearlabels(sceneobj, groups, clearcat, sampref)
%
%  Clears internal assignments of data to classes for specified groups
%  (by default, all).  Useful to test learning from synthetic data.
%  * clearcat flag => also clears object category labels (default=true).
%  * sampref flag  => samples reference positions from prior (default=false).

  % Default parameters
  if nargin < 2
    groups = [];
  end
  if nargin < 3
    clearcat = true;
  end
  if nargin < 4
    sampref = false;
  end

  if isempty(groups)
    groups = [1:sceneobj.numgroup];
  end

  % Hyperparameters
  dtype    = sceneobj.dtype;
  dparam   = sceneobj.dparam;
  numtype  = sceneobj.numtype;
  numobj   = sceneobj.numobj;

  useref   = sceneobj.useref;
  if useref
    dim = sceneobj.rparam(1);
  end

  % Class (part) state variables
  classqq = sceneobj.classqq;
  classnd = sceneobj.classnd;
  objnd   = sceneobj.objnd;
  scenend = sceneobj.scenend;

  % Clear labels for each specified group
  for j = 1:length(groups)
    jj = groups(j);

    % State variables for current group
    numdata  = sceneobj.numdata(jj);
    datass   = sceneobj.datass{jj};
    datacc   = sceneobj.datacc{jj};
    dataoo   = sceneobj.dataoo{jj};
    labeloo  = sceneobj.labeloo{jj};
    category = sceneobj.category(jj);

    % Clear reference position from statistics
    if useref
      groupref = sceneobj.groupmean(:,jj);

      sceneobj.groupmean(:,jj)       = 0;
      sceneobj.refsum(:,category)    = sceneobj.refsum(:,category) - groupref;
      sceneobj.refchol(:,:,category) = ...
        cholupdate(sceneobj.refchol(:,:,category), groupref, '-');

      groupnd  = sceneobj.groupnd(:,:,jj);
      groupsum = sceneobj.groupsum(:,:,:,jj);
    end

    % Remove each data item from cached statistics
    for ii = find(datacc > 0)
      for dd = 1:numtype
        if useref && dd==1
          delss = datass{dd}(:,ii) - groupref(dim*(dataoo(ii)-1)+1:dim*dataoo(ii));
          classqq{dd}(:,datacc(ii)) = datastat_del(classqq{dd}(:,datacc(ii)), ...
            classnd(datacc(ii)), delss, dtype(dd), dparam{dd});
          groupnd(dataoo(ii),datacc(ii)) = groupnd(dataoo(ii),datacc(ii)) - 1;
          groupsum(:,dataoo(ii),datacc(ii)) = ...
            groupsum(:,dataoo(ii),datacc(ii)) - datass{dd}(:,ii);
        else
          classqq{dd}(:,datacc(ii)) = datastat_del(classqq{dd}(:,datacc(ii)), ...
            classnd(datacc(ii)), datass{dd}(:,ii), dtype(dd), dparam{dd});
        end
      end

      classnd(datacc(ii)) = classnd(datacc(ii)) - 1;
      objnd(dataoo(ii),datacc(ii)) = objnd(dataoo(ii),datacc(ii)) - 1;
      scenend(dataoo(ii),category) = scenend(dataoo(ii),category) - 1; 
    end

    if useref
      sceneobj.groupnd(:,:,jj)    = groupnd;
      sceneobj.groupsum(:,:,:,jj) = groupsum;

      % Optionally resample reference position from prior
      if sampref
        refnu    = sceneobj.rparam(2) + sceneobj.sceneng(category);
        refkappa = sceneobj.rparam(3) + sceneobj.sceneng(category);
        refmean  = sceneobj.refsum(:,category) / refkappa;
        refcovar = cholupdate(sceneobj.refchol(:,:,category), ...
          sceneobj.refsum(:,category) / sqrt(refkappa), '-');
        refcovar = refcovar'*refcovar * ((refkappa+1)/(refkappa*(refnu-numobj*dim-2)));

        newref = refmean + chol(refcovar)'*randn(numobj*dim,1);

        sceneobj.groupmean(:,jj)       = newref;
        sceneobj.refsum(:,category)    = sceneobj.refsum(:,category) + newref;
        sceneobj.refchol(:,:,category) = ...
          cholupdate(sceneobj.refchol(:,:,category), newref);
      end
    end

    % Clear assignments of data items to classes
    sceneobj.datacc{jj} = zeros(1,numdata);

    % If desired, clear object category labels
    if clearcat
      sceneobj.labeloo{jj} = zeros(1,numdata);
    end
    sceneobj.dataoo{jj} = sceneobj.labeloo{jj};
  end

  sceneobj.classqq = classqq;
  sceneobj.classnd = classnd;
  sceneobj.objnd   = objnd;
  sceneobj.scenend = scenend;

