
function catprob = estcategory(sceneobj, groups)
%estcategory    Posterior scene category log-likelihoods for current assignments
%
%    catprob = estcategory(SO, groups)
%
%  Compute posterior scene category log-likelihoods implied by current
%  assignments of data to objects and classes, where
%    groups  = groups for which probabilites computed (all, by default)
%    catprob = (numscene x length(groups)) matrix of these probabilities

  % default parameters
  if nargin < 2
    groups = [1:sceneobj.numgroup];
  end

  % static parameters
  gamma    = sceneobj.gamma;
  alpha    = sceneobj.alpha;
  numscene = sceneobj.numscene;
  numobj   = sceneobj.numobj;
  numclass = sceneobj.numclass;
  numtype  = sceneobj.numtype;
  dtype    = sceneobj.dtype;
  dparam   = sceneobj.dparam;
  useref   = sceneobj.useref;

  % class (part) level state variables
  classnd = sceneobj.classnd;
  classqq = sceneobj.classqq;

  % compute P(object|scene)
  objprob = sceneobj.scenend + gamma;
  objnorm = sum(objprob,1);
  objprob = objprob ./ repmat(objnorm,numobj,1);

  % compute P(class|object)
  classprob = sceneobj.objnd + alpha;
  classnorm = sum(classprob,2);
  classprob = classprob ./ repmat(classnorm,1,numclass);

  % initialize output data structure
  numgroup = size(groups,2);
  catprob  = zeros(numscene,numgroup);

  for j = 1:numgroup
    jj = groups(j);

    % data for current group
    numdata = sceneobj.numdata(jj);
    datass  = sceneobj.datass{jj};

    % compute logP(feature|class,object)
    clik = zeros(numdata, numobj, numclass);
    if useref
      startind = 2;
  
      dim   = dparam{1}(1);
      theta = datastat_estparam(classqq{1}, classnd, dtype(1), dparam{1});
  
      classmean  = theta(1:dim,:);
      classcovar = reshape(theta(dim+1:end,:), dim, dim, numclass);
      groupmean  = sceneobj.groupmean(:,jj);
      groupcovar = sceneobj.groupcovar(:,:,jj);
  
      for oo = 1:numobj
        indA = dim*(oo-1) + 1;
        indB = dim*oo;
  
        for cc = 1:numclass
          localmean     = groupmean(indA:indB) + classmean(:,cc);
          localcovar    = groupcovar(indA:indB,indA:indB) + classcovar(:,:,cc);
          clik(:,oo,cc) = -0.5*log(det(localcovar));
  
          for ii = 1:numdata
            delss = datass{1}(:,ii) - localmean;
            clik(ii,oo,cc) = clik(ii,oo,cc) - 0.5* delss' * (localcovar \ delss);
          end
        end
      end
    else
      startind = 1;
    end
  
    for dd = startind:numtype
      classqqLocal = classqq{dd};
      datassLocal  = datass{dd};
      dtypeLocal   = dtype(dd);
      dparamLocal  = dparam{dd};
  
      for ii = 1:numdata
        datallik = datastat_condlike(classqqLocal, classnd, datassLocal(:,ii), ...
          dtypeLocal, dparamLocal);
        clik(ii,:,:) = clik(ii,:,:) + ...
          repmat(reshape(datallik, [1 1 numclass]), [1 numobj 1]);
      end
    end

    % compute P(feature|scene)
    posterior = exp(clik) .* ...
      repmat(reshape(classprob, [1 numobj numclass]), [numdata 1 1]);
    featprob  = sum(posterior,3) * objprob; 

    % compute unnormalized log-probability to allow comparison to other models
    catprob(:,j) = sum(log(featprob),1)';   
  end

