
function sceneobj = gibbs_refSamp(sceneobj, groups)
%gibbs_refSamp     Iteration of Gibbs sampler, also sampling reference positions

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if sceneobj.numtype > 2
  error('Gibbs sampler supports at most two data types');
end
usetypeB = (sceneobj.numtype > 1);

% verify that first datatype is Gaussian, to match reference position code
if sceneobj.dtype(1) ~= 2
  error('Reference position EM updates assume first datatype is Gaussian');
end

% static parameters
alpha    = sceneobj.alpha;
gamma    = sceneobj.gamma;
dtype    = sceneobj.dtype;
dparam   = sceneobj.dparam{1};
numobj   = sceneobj.numobj;
numclass = sceneobj.numclass;

% reference position parameters
dim      = sceneobj.rparam(1);

% class position parameters
classnu    = dparam(2);
classkappa = dparam(3);

% class (part) level state variables
classnd = sceneobj.classnd;
classqq = sceneobj.classqq{1};
objnd   = sceneobj.objnd;

% parameters for second datatype
if usetypeB
  dparamB  = sceneobj.dparam{2};
  classqqB = sceneobj.classqq{2};
end

% temporary variables used in reference transformation resampling
classprecis = zeros(dim,dim,numclass);
classmean   = zeros(dim,numclass);

% resample groups in random order
for j = randperm(length(groups))
  jj = groups(j);

  % state variables for current group
  numdata  = sceneobj.numdata(jj);
  datass   = sceneobj.datass{jj}{1};
  datacc   = sceneobj.datacc{jj};
  dataoo   = sceneobj.dataoo{jj};
  labeloo  = sceneobj.labeloo{jj};
  category = sceneobj.category(jj);
  scenend  = sceneobj.scenend(:,category);
  refnu    = sceneobj.rparam(2) + sceneobj.sceneng(category);
  refkappa = sceneobj.rparam(3) + sceneobj.sceneng(category);

  % state variables for second datatype
  if usetypeB
    datassB = sceneobj.datass{jj}{2};
  else
    datassB = zeros(0,numdata);
  end

  % group (image) state variables for reference position estimation
  groupnd    = sceneobj.groupnd(:,:,jj);
  groupsum   = sceneobj.groupsum(:,:,:,jj);
  groupmean  = sceneobj.groupmean(:,jj);

  % reference position state variables 
  refsum     = sceneobj.refsum(:,category);
  refchol    = sceneobj.refchol(:,:,category);

  % sample part associated with each data item, in random order
  for ii = randperm(numdata)
    ss    = datass(:,ii);
    ssB   = datassB(:,ii);
    oldcc = datacc(ii);
    oldoo = dataoo(ii);

    % temporarily remove data item from model, unless not yet assigned
    if oldcc > 0
      ssref = ss - groupmean(dim*(oldoo-1)+1:dim*oldoo);
      classqq(:,oldcc) = datastat_del(classqq(:,oldcc), ...
        classnd(oldcc), ssref, dtype(1), dparam);
      groupsum(:,oldoo,oldcc) = groupsum(:,oldoo,oldcc) - ss;

      if usetypeB
        classqqB(:,oldcc) = datastat_del(classqqB(:,oldcc), ...
          classnd(oldcc), ssB, dtype(2), dparamB);
      end

      classnd(oldcc)       = classnd(oldcc) - 1;
      objnd(oldoo,oldcc)   = objnd(oldoo,oldcc) - 1;
      scenend(oldoo)       = scenend(oldoo) - 1;
      groupnd(oldoo,oldcc) = groupnd(oldoo,oldcc) - 1;
    end

    % compute conditional likelihood factors for each class 
    if usetypeB
      clik = datastat_condlike(classqqB, classnd, ssB, dtype(2), dparamB);
    else
      clik = zeros(1,numclass);
    end

    % if object unlabelled, compute probability of every object-part pair
    % otherwise, compute probability of each part given known object
    % account for scene and object-level Dirichlet priors
    if labeloo(ii) > 0
      newoo = labeloo(ii);
      ssref = ss - groupmean(dim*(newoo-1)+1:dim*newoo);

      clik  = clik + datastat_condlike(classqq, classnd, ssref, dtype(1), dparam);
      clik  = exp(clik-max(clik)) .* (objnd(newoo,:) + alpha);
      newcc = multinomial_rnd(clik, 1);
    else
      prob = clik(ones(numobj,1),:);  % prob = repmat(clik,numobj,1);
      for oo = 1:numobj
        ssref = ss - groupmean(dim*(oo-1)+1:dim*oo);
        prob(oo,:) = prob(oo,:) + ...
          datastat_condlike(classqq, classnd, ssref, dtype(1), dparam);
      end

      prob     = exp(prob-max(max(prob))) .* (objnd + alpha);
      probnorm = (scenend+gamma) ./ sum(objnd+alpha,2);
      prob     = prob .* probnorm(:,ones(1,numclass));

      % allow exclusion of single object category
      if labeloo(ii) < 0
        prob(-labeloo(ii),:) = 0;
      end

      newoocc = multinomial_rnd(prob(:),1);
      newcc   = fix((newoocc-1)/numobj) + 1;
      newoo   = newoocc - (newcc-1)*numobj;
    end

    % add data item back to model
    datacc(ii) = newcc;
    dataoo(ii) = newoo;

    ssref = ss - groupmean(dim*(newoo-1)+1:dim*newoo);
    classqq(:,newcc) = datastat_add(classqq(:,newcc), ...
      classnd(newcc), ssref, dtype(1), dparam);
    groupsum(:,newoo,newcc) = groupsum(:,newoo,newcc) + ss;

    if usetypeB
      classqqB(:,newcc) = datastat_add(classqqB(:,newcc), ...
        classnd(newcc), ssB, dtype(2), dparamB);
    end

    classnd(newcc)       = classnd(newcc) + 1;
    objnd(newoo,newcc)   = objnd(newoo,newcc) + 1;
    scenend(newoo)       = scenend(newoo) + 1;
    groupnd(newoo,newcc) = groupnd(newoo,newcc) + 1;
  end

  % posterior part distributions corresponding to current assignments
  thetaqq     = datastat_estparam(classqq, classnd, dtype(1), dparam);
  classmean   = thetaqq(1:dim,:);
  classprecis = zeros(dim,dim,numclass);
  for cc = 1:numclass
    classprecis(:,:,cc) = inv(reshape(thetaqq(dim+1:end,cc),dim,dim));
  end

  % remove current group from reference position statistics
  refsum   = refsum - groupmean;
  refchol  = cholupdate(refchol, groupmean, '-');
  refmean  = refsum / refkappa;
  refcovar = cholupdate(refchol, refsum / sqrt(refkappa), '-');
  refcovar = refcovar'*refcovar * ((refkappa+1)/(refkappa*(refnu-numobj*dim-2)));

  % compute posterior reference position distribution given feature assignments
  oldmean    = groupmean;
  groupmean  = zeros(numobj*dim,1);
  groupcovar = inv(refcovar);
  for oo = 1:numobj
    indA = dim*(oo-1) + 1;
    indB = dim*oo;

    for cc = 1:numclass
      groupcovar(indA:indB,indA:indB) = groupcovar(indA:indB,indA:indB) + ...
        groupnd(oo,cc)*classprecis(:,:,cc);
      groupmean(indA:indB) = groupmean(indA:indB) + classprecis(:,:,cc) * ...
        (groupsum(:,oo,cc) - groupnd(oo,cc)*(classmean(:,cc)+refmean(indA:indB)));
    end
  end
  groupcovar = inv(groupcovar);
  groupmean  = refmean + groupcovar*groupmean;

  % sample a new reference transformation
  newmean    = groupmean + chol(groupcovar)'*randn(numobj*dim,1);

  % add current group back to reference position statistics
  refsum  = refsum + newmean;
  refchol = cholupdate(refchol, newmean);

  % update class position statistics
  classsum = classqq(1:dim,:);
  classout = reshape(classqq(dim+1:end-1,:),dim,dim,numclass); 
  for cc = 1:numclass
    classout(:,:,cc) = cholupdate(classout(:,:,cc), ...
      classsum(:,cc)./sqrt(classkappa+classnd(cc)));
    classout(:,:,cc) = classout(:,:,cc)'*classout(:,:,cc);
  end

  delmean  = newmean - oldmean;
  for oo = 1:numobj
    indA = dim*(oo-1) + 1;
    indB = dim*oo;

    for cc = 1:numclass
      classsum(:,cc)   = classsum(:,cc) - groupnd(oo,cc)*delmean(indA:indB);
      classout(:,:,cc) = classout(:,:,cc)-groupsum(:,oo,cc)*delmean(indA:indB)';
      classout(:,:,cc) = classout(:,:,cc)-delmean(indA:indB)*groupsum(:,oo,cc)';
      classout(:,:,cc) = classout(:,:,cc) + groupnd(oo,cc) * ...
        (newmean(indA:indB)*newmean(indA:indB)' - ...
         oldmean(indA:indB)*oldmean(indA:indB)');
    end
  end

  covnorm = (classkappa+classnd+1)./((classkappa+classnd).*(classnu+classnd-dim-1));
  for cc = 1:numclass
    classout(:,:,cc) = chol(classout(:,:,cc));
    classout(:,:,cc) = cholupdate(classout(:,:,cc), ...
      classsum(:,cc)./sqrt(classkappa+classnd(cc)), '-');
    classqq(end,cc)  = -log(covnorm(cc)) - sum(log(diag(classout(:,:,cc))));
  end
  classqq(1:dim,:) = classsum;
  classqq(dim+1:end-1,:) = reshape(classout, dim^2, numclass);

  % cache all state variables associated only with current group (image)
  sceneobj.datacc{jj}            = datacc;
  sceneobj.dataoo{jj}            = dataoo;
  sceneobj.scenend(:,category)   = scenend;
  sceneobj.groupnd(:,:,jj)       = groupnd;
  sceneobj.groupsum(:,:,:,jj)    = groupsum;
  sceneobj.groupmean(:,jj)       = newmean;
  sceneobj.refsum(:,category)    = refsum;
  sceneobj.refchol(:,:,category) = refchol;
end

% store class state variables for next iteration
sceneobj.classqq{1} = classqq;
sceneobj.classnd    = classnd;
sceneobj.objnd      = objnd; 
if usetypeB
  sceneobj.classqq{2} = classqqB;
end

