
function [sceneobj,probpart,partparam] = ...
  sampledata(sceneobj, groups, scene, numdata, probobj, probpart, partparam, refparam)
%sampledata     Generate random datasets using specified, or random, parameters
%
%    SO = sampledata(SO, groups, scene, numdata, ...
%                    probobj, probclass, classparam, refparam)
%
%  Generate synthetic dataset by random sampling, where
%    groups     = groups for which synthetic data created (all, by default)
%    scene      = scene type for specified groups, or 0 to sample
%                 uniformly among scenes for each group (default)
%    numdata    = number of new data items for each group
%    probscene  = (numscene x numobj) matrix of P(category|scene), or empty
%                 to sample this randomly from Dirichlet prior
%    probclass  = (numobj x numclass) matrix of P(class|category), or empty
%                 to sample this randomly from Dirichlet prior
%    classparam = parameters for each class, or empty to randomly sample
%                 using densities implied by data hyperparameters
%    refparam   = reference position parameters (one column per scene type),
%                 or empty to sample using reference hyperparameters

  % Default parameters
  if nargin < 2
    groups = [];
  end
  if nargin < 3
    scene = 0;
  end
  if nargin < 4
    numdata = 100;
  end
  if nargin < 5
    probobj = [];
  end
  if nargin < 6
    probpart = [];
  end
  if nargin < 7
    partparam = {};
  end
  if nargin < 8
    refparam = [];
  end

  if ~iscell(partparam)
    partparam = {partparam};
  end

  if isempty(groups)
    groups = [1:sceneobj.numgroup];
  end

  % Hyperparameters
  numscene = sceneobj.numscene;
  numobj   = sceneobj.numobj;
  numtype  = sceneobj.numtype;
  numclass = sceneobj.numclass;
  alpha    = sceneobj.alpha;
  gamma    = sceneobj.gamma;
  dtype    = sceneobj.dtype;
  dparam   = sceneobj.dparam;
  useref   = sceneobj.useref;

  % Class (part) state variables
  classqq = sceneobj.classqq;
  classnd = sceneobj.classnd;
  objnd   = sceneobj.objnd;
  scenend = sceneobj.scenend;

  % Determine scene labels and p(object|scene)
  if scene == 0
    datacat = multinomial_rnd(ones(numscene,1), length(groups));
  else
    datacat = scene*ones(length(groups),1);
  end

  if isempty(probobj)
    probobj = dirichlet_rnd(gamma*ones(numobj,1), numscene)';
  elseif size(probobj,1) == 1
    probobj = repmat(probobj,numscene,1);
  end

  % Determine p(part|object)
  if isempty(probpart)
    probpart = dirichlet_rnd(alpha*ones(numclass,1), numobj)';
  elseif size(probpart,1) == 1
    probpart = repmat(probpart,numobj,1);
  end

  % Determine parameters associated with each part
  if isempty(partparam)
    for dd = 1:numtype
      for cc = 1:numclass
        partparam{dd}(:,cc) = datastat_sampparam(classqq{dd}(:,cc), ...
          classnd(cc), 1, dtype(dd), dparam{dd});
      end
    end
  else
    for dd = 1:numtype 
      if size(partparam{dd},2) ~= numclass
        error('Improper number of part parameters');
      end
    end
  end

  % Determine reference position parameters associated with each scene
  if useref
    dim   = sceneobj.rparam(1);
    nu    = sceneobj.rparam(2);
    kappa = sceneobj.rparam(3);

    if isempty(refparam)
      mu    = sceneobj.refsum(:,1) ./ kappa;
      scale = sceneobj.refchol(:,:,1)'*sceneobj.refchol(:,:,1);

      for aa = 1:numscene
        refcovar(:,:,aa) = invwishart_rnd(scale, nu, 1);
        refmean(:,aa)    = mu + chol(refcovar(:,:,aa)./kappa)'*randn(numobj*dim,1);
      end
    else
      refmean  = refparam(1:numobj*dim,:);
      refcovar = reshape(refparam(numobj*dim+1:end,:), ...
        numobj*dim, numobj*dim, numscene);
    end
  end

  % Sample dataset for each specified group
  for j = 1:length(groups)
    jj = groups(j);

    % Check consistency of scene category label
    if (sceneobj.category(jj) > 0) && (datacat(j) ~= sceneobj.category(jj))
      error('Supplied category label does not match this group');
    end
    if sceneobj.category(jj) == 0
      sceneobj.sceneng(datacat(j)) = sceneobj.sceneng(datacat(j)) + 1;
    end
    sceneobj.category(jj) = datacat(j);

    % Sample reference position
    if useref
      groupref = refmean(:,datacat(j)) + ...
        chol(refcovar(:,:,datacat(j)))' * randn(numobj*dim,1);

      sceneobj.groupmean(:,jj)       = groupref;
      sceneobj.refsum(:,datacat(j))  = sceneobj.refsum(:,datacat(j)) + groupref;
      sceneobj.refchol(:,:,datacat(j)) = ...
        cholupdate(sceneobj.refchol(:,:,datacat(j)), groupref);

      groupnd  = sceneobj.groupnd(:,:,jj);
      groupsum = sceneobj.groupsum(:,:,:,jj);
    end
    
    % Independently sample each data item
    newdatass = cell(numtype,1);
    newdatacc = zeros(1,numdata);
    newdataoo = zeros(1,numdata);
    for ii = 1:numdata
      % sample object and part associated with data item
      newdataoo(ii) = multinomial_rnd(probobj(datacat(j),:), 1);
      newdatacc(ii) = multinomial_rnd(probpart(newdataoo(ii),:), 1);

      % sample data item using part parameters, and update part statistics
      for dd = 1:numtype
        newdatass{dd}(:,ii) = dataparam_sampdata(partparam{dd}(:,newdatacc(ii)),...
          1, dtype(dd), dparam{dd});
        classqq{dd}(:,newdatacc(ii)) = datastat_add(classqq{dd}(:,newdatacc(ii)),...
          classnd(newdatacc(ii)), newdatass{dd}(:,ii), dtype(dd), dparam{dd});
      end

      if useref 
        newdatass{1}(:,ii) = newdatass{1}(:,ii) + ...
          groupref(dim*(newdataoo(ii)-1)+1:dim*newdataoo(ii));
        groupnd(newdataoo(ii),newdatacc(ii)) = ...
          groupnd(newdataoo(ii),newdatacc(ii)) + 1;
        groupsum(:,newdataoo(ii),newdatacc(ii)) = ...
          groupsum(:,newdataoo(ii),newdatacc(ii)) + newdatass{1}(:,ii);
      end

      classnd(newdatacc(ii)) = classnd(newdatacc(ii)) + 1;
      objnd(newdataoo(ii),newdatacc(ii)) = objnd(newdataoo(ii),newdatacc(ii)) + 1;
      scenend(newdataoo(ii),datacat(j))  = scenend(newdataoo(ii),datacat(j)) + 1;
    end

    % Store data and generating class assignments
    sceneobj.numdata(jj) = sceneobj.numdata(jj) + numdata;
    sceneobj.datacc{jj}  = [sceneobj.datacc{jj}, newdatacc];
    sceneobj.dataoo{jj}  = [sceneobj.dataoo{jj}, newdataoo];
    sceneobj.labeloo{jj} = [sceneobj.labeloo{jj}, newdataoo];
    for dd = 1:numtype
      sceneobj.datass{jj}{dd} = [sceneobj.datass{jj}{dd}, newdatass{dd}];
    end

    if useref
      sceneobj.groupnd(:,:,jj)    = groupnd;
      sceneobj.groupsum(:,:,:,jj) = groupsum;
    end
  end

  sceneobj.classqq = classqq;
  sceneobj.classnd = classnd;
  sceneobj.objnd   = objnd;
  sceneobj.scenend = scenend;

