
function visualize(sceneobj, plotstyle, Hfig)
%visualize     Visualize posterior distributions implied by current assignments
%
%    visualize(SO, plotstyle, Hfig)
%
%  Visualize classes implied by current assignments, where plotstyle is
%  a cell array of structs of datatype-specific parameters, and Hfig
%  is a vector of figure handles for each datatype (if empty, a new
%  figure is created for each datatype).
%  Overlays reference positions on Gaussian plot, if used.

if nargin < 2
  plotstyle = struct([]);
end
if nargin < 3
  Hfig = [];
end

dtype    = sceneobj.dtype;
dparam   = sceneobj.dparam;
numobj   = sceneobj.numobj;
numtype  = sceneobj.numtype;
numclass = sceneobj.numclass;
classqq  = sceneobj.classqq;
classnd  = sceneobj.classnd;
alpha    = sceneobj.alpha;
objnd    = sceneobj.objnd;

if ~iscell(plotstyle)
  tempstyle = plotstyle;
  plotstyle = cell(numtype,1);

  for dd = 1:numtype
    plotstyle{dd} = tempstyle;
  end
end

if isempty(Hfig)
  for dd = 1:numtype
    Hfig(dd) = figure;
  end
end

% Posterior class probabilities for each object category
classprob = objnd' + alpha;
Z = sum(classprob,1);
classprob = classprob ./ repmat(Z,numclass,1);

for dd = 1:numtype
  % Posterior parameters for each class
  theta = datastat_estparam(classqq{dd}, classnd, dtype(dd), dparam{dd});

  switch dtype(dd)
  case 1,   % Discrete multinomial
    Hax = subplot(1,3,1,'Parent',Hfig(dd));
    imagesc(theta*classprob,'Parent',Hax);
    colormap(Hax,gray(256));
    title(Hax,'Prob(feature|category)');
    
    Hax = subplot(1,3,2,'Parent',Hfig(dd));
    imagesc(theta,'Parent',Hax);
    colormap(Hax,gray(256));
    title(Hax,'Prob(feature|part)');
  
    Hax = subplot(1,3,3,'Parent',Hfig(dd));
    imagesc(classprob,'Parent',Hax);
    colormap(Hax,gray(256));
    title(Hax,'Prob(part|category)');
  
  case 2,   % Normal-Inverse-Wishart
    Hax = subplot(2,1,1,'Parent',Hfig(dd));
    imagesc(classprob,'Parent',Hax);
    colormap(Hax,gray(256));
    title(Hax,'Prob(part|category)');
  
    % Posterior means and variances for each class
    dim   = dparam{dd}(1);
    mu    = theta(1:dim,:);
    sigma = reshape(theta(dim+1:end,:),dim,dim,numclass);
  
    Hax = subplot(2,1,2,'Parent',Hfig(dd));
    cla(Hax);
    if isfield(plotstyle{dd}, 'numbers') && (plotstyle{dd}.numbers > 0)
      plotGaussMix(mu, sigma, classnd, 'numbers', Hax);
    else
      plotGaussMix(mu, sigma, classnd, [], Hax);
    end

    % Overlay posterior reference positions
    if sceneobj.useref
      % Statistics for first scene
      if sceneobj.numscene > 1
        warning('Only plotting reference position posterior for first scene');
      end
      category = 1;
      scenend  = sceneobj.scenend(:,category);
      refsum   = sceneobj.refsum(:,category);
      refchol  = sceneobj.refchol(:,:,category);
      nu      = sceneobj.rparam(2) + sceneobj.sceneng(category);
      kappa   = sceneobj.rparam(3) + sceneobj.sceneng(category);

      % Compute current posterior mean and covariance
      refmean  = refsum / kappa;
      refcovar = cholupdate(refchol, refsum / sqrt(kappa), '-');
      refcovar = refcovar'*refcovar * ((kappa+1)/(kappa*(nu+numobj*dim+1)));

      % Plot marginal for each object category
      muref  = reshape(refmean,dim,numobj);
      sigref = zeros(dim,dim,numobj);
      for oo = 1:numobj
        sigref(:,:,oo) = refcovar(dim*(oo-1)+1:dim*oo,dim*(oo-1)+1:dim*oo);
      end

      hold(Hax,'on');
      plotGaussMix(muref, sigref, scenend, 'g', Hax);
    end

    if isfield(plotstyle{dd}, 'range')
      axis(Hax, plotstyle{dd}.range);
    end

  case 3,   % Diagonal Normal-Inverse-Wishart
    Hax = subplot(1,3,3,'Parent',Hfig(dd));
    imagesc(classprob,'Parent',Hax);
    colormap(Hax,gray(256));
    title(Hax,'Prob(part|category)');
  
    % Posterior means and variances for each class
    dim   = dparam{dd}(1);
    mu    = theta(1:dim,:);
    sigma = theta(dim+1:end,:);  

    Hax = subplot(1,3,1,'Parent',Hfig(dd));
    if isfield(plotstyle{dd}, 'meanscale')
      imagesc(mu,'Parent',Hax,plotstyle{dd}.meanscale);
    else
      imagesc(mu,'Parent',Hax);
    end
    colormap(Hax,gray(256));
    title(Hax,'Mean(feature|part)');

    Hax = subplot(1,3,2,'Parent',Hfig(dd));
    if isfield(plotstyle{dd}, 'stdevscale')
      imagesc(sqrt(sigma),'Parent',Hax,plotstyle{dd}.stdevscale);
    else
      imagesc(sqrt(sigma),'Parent',Hax);
    end
    colormap(Hax,gray(256));
    title(Hax,'Stdev(feature|part)');

  end
end

