
function tdp = clearassign(tdp, groups, labels)
%clearassign    Clear assignments of data to tables for specified groups
%
%    TDP = clearassign(TDP, groups, labels)
%
%  Clears internal assignments of data to tables and classes, where
%    groups = groups for assignments cleared (by default, all)
%    labels = list of labels which data items must match to be cleared,
%             or empty to clear all assignments (default)
%  Useful to test learning from synthetic data, or to run Gibbs sampling
%  with multiple restarts on test data.

% default parameters
if nargin < 2
  groups = [];
end
if nargin < 3
  labels = [];
end

numgroup = tdp.numgroup;
if isempty(groups)
  groups = [1:numgroup];
end

% class (part) level state variables
numclass = tdp.numclass;
classnd  = tdp.classnd;
classnt  = tdp.classnt;
classpp  = tdp.classpp;
classqq  = tdp.classqq;

% data parameters
numtype = tdp.numtype;
dtype   = tdp.dtype;
dparam  = tdp.dparam;

% transformation parameters
ttype   = tdp.ttype;
tparam  = tdp.tparam;

% clear assignments for each specified group
for j = 1:length(groups)
  jj = groups(j);

  % table and data level state variables for current group
  numdata  = tdp.numdata(jj);
  numtable = tdp.numtable(jj);
  datass   = tdp.datass{jj};
  datatt   = tdp.datatt{jj};
  datall   = tdp.datall{jj};
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % remove all assigned data items from cached statistics
  if isempty(labels)
    clearind = find(datatt > 0);
  else
    clearind = find((datatt > 0) & ismember(datall,labels));
  end
  for ii = clearind
    oldtt = datatt(ii);
    oldcc = tablecc(oldtt);

    for dd = 1:numtype
      %TODO: support generic transformations
      if ttype > 0 && dd == 1
        ssref = datass{dd}(:,ii) - tablerr(:,oldtt);
        classqq{dd}(:,oldcc) = datastat_del(classqq{dd}(:,oldcc), ...
          classnd(oldcc), ssref, dtype(dd), dparam{dd});
      else
        classqq{dd}(:,oldcc) = datastat_del(classqq{dd}(:,oldcc), ...
          classnd(oldcc), datass{dd}(:,ii), dtype(dd), dparam{dd});
      end
    end
    classnd(oldcc) = classnd(oldcc) - 1;
    tablend(oldtt) = tablend(oldtt) - 1;
  end
  datatt(clearind) = 0;

  % remove deserted tables
  for tt = numtable:-1:1
    if tablend(tt) == 0
      numtable = numtable - 1;
      cc = tablecc(tt);
      classpp(:,cc) = transtat_del(classpp(:,cc), ...
        classnt(cc), tablerr(:,tt), ttype, tparam);
      classnt(cc)   = classnt(cc) - 1;

      ii = find(datatt > tt);
      datatt(ii) = datatt(ii) - 1;
      tablecc(tt) = [];
      tablerr(:,tt) = [];
      tablend(tt) = [];
    end
  end

  % store updated table statistics
  tdp.numtable(jj) = numtable;
  tdp.datatt{jj}   = datatt;
  tdp.tablecc{jj}  = tablecc;
  tdp.tablerr{jj}  = tablerr;
  tdp.tablend{jj}  = tablend;
end

% remove any unused classes
for cc = numclass:-1:1
  if classnt(cc) == 0
    numclass      = numclass - 1;
    classnd(cc)   = [];
    classnt(cc)   = [];
    classpp(:,cc) = [];
    for dd = 1:numtype
      classqq{dd}(:,cc) = [];
    end

    for jj = 1:numgroup
      tt = find(tdp.tablecc{jj} > cc);
      tdp.tablecc{jj}(tt) = tdp.tablecc{jj}(tt) - 1;
    end
  end
end

% store updated classes
tdp.numclass = numclass;
tdp.classnd  = classnd;
tdp.classnt  = classnt;
tdp.classpp  = classpp;
tdp.classqq  = classqq;

