
function tdp = doGibbs(tdp, numiter, visparam, sampgroup, samplabel, coniter)
%doGibbs     Approximates posterior distributions via Gibbs sampling
%
%    TDP = doGibbs(TDP, numiter, visparam, sampgroup, samplabel, coniter)
%
%  Learning and inference via Gibbs sampling, where
%    numiter   = number of iterations (one iteration resamples all 
%                assignments of data to classes, in all chosen groups, once)
%    visparam  = cell array of arguments to visualize, or empty for
%                no plot of learning process
%    sampgroup = groups on which to run Gibbs sampling (by default, all)
%    samplabel = vector of data labels to sample, or empty to sample all
%                data assignments in sampgroup regardless of label (default),
%                or cell array giving different labels for each of sampgroup
%    coniter   = number of concentration parameter resampling iterations
%                between each Gibbs sampling iteration (by default, zero)

  % default parameters
  if nargin < 2
    numiter = 1;
  end
  if nargin < 3
    visparam = [];
  end
  if nargin < 4 
    sampgroup = [1:tdp.numgroup];
  end
  if nargin < 5
    samplabel = [];
  end
  if nargin < 6
    coniter = 0;
  end

  % process specified data labels
  if ~iscell(samplabel)
    inputlabel = samplabel;
    samplabel  = cell(length(sampgroup),1);
    [samplabel{:}] = deal(inputlabel);
  end
  for j = 1:length(sampgroup)
    jj = sampgroup(j);

    if isempty(samplabel{j})
      samplabel{j} = unique(tdp.datall{jj});
    end
  end

  for iter = 1:numiter
    % table and global cluster assignments
    tdp = gibbs_crf(tdp, sampgroup, samplabel);

    % concentration parameters
    if coniter > 0
      if isfinite(tdp.gamma)
        tdp.gamma = gibbs_conparam(tdp.gamma, sum(tdp.numtable), tdp.numclass, ...
          tdp.gammaprior(1), tdp.gammaprior(2), coniter);
      end

      % must only include groups where data has been assigned
      validgroup = find(tdp.numtable > 0);  
      tdp.alpha  = gibbs_conparam(tdp.alpha, tdp.numdata(validgroup), ...
        tdp.numtable(validgroup), tdp.alphaprior(1), tdp.alphaprior(2), coniter);
    end

    if ~isempty(visparam)
      visualize(tdp, visparam{:});
      drawnow;
    end

    fprintf(1,'%d(%d).',iter,tdp.numclass);
  end
  fprintf(1,'\n');

