
function testprob = estassign(tdp, group, testdata)
%estassign     Posterior probability of transformed classes generating given data
%
%    testprob = estassign(TDP, group, testdata)
%
%  Posterior probability that each class generated each data point, where
%    group    = single group for which data assignment probabilities computed
%    testdata = matrix such that each column is a test data point,
%               or if multiple datatypes a cell array such that
%               newdata{dd} is the data matrix for datatype dd
%    testprob = (numdata x numclass+1) matrix P(class|data,group) (ttype=0), or
%               (numdata x numtable+1) matrix P(table|data,group) (ttype>0)

if ~iscell(testdata)
  testdata = {testdata};
end

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if tdp.numtype > 2
  error('Posterior assignment evaluation supports at most two data types');
end
usetypeB = (tdp.numtype > 1);

numtest = size(testdata{1},2);
testss  = testdata{1};
if usetypeB
  testssB = testdata{2};
else
  testssB = zeros(0,numtest);
end

% class (part) level state variables
numclass = tdp.numclass;
classnd  = tdp.classnd;
classqq  = tdp.classqq{1};

% data parameters
numtype = tdp.numtype;
dtype   = tdp.dtype;
dparam  = tdp.dparam{1};
qq0     = tdp.qq0{1};

% parameters and state variables for second datatype
if usetypeB
  dparamB  = tdp.dparam{2};
  qq0B     = tdp.qq0{2};
  classqqB = tdp.classqq{2};
end

% transformation parameters
ttype    = tdp.ttype;
tparam   = tdp.tparam;
pp0      = tdp.pp0;

% table state variables for specified group
numtable = tdp.numtable(group);
tablecc  = tdp.tablecc{group};
tablerr  = tdp.tablerr{group};
tablend  = tdp.tablend{group};

% add empty class to allow evaluation of likelihood for undefined clusters
classnd(numclass+1)   = 0;
classqq(:,numclass+1) = qq0; 
if usetypeB
  classqqB(:,numclass+1) = qq0B;
end

% evaluate log-likelihood of each data point, given each global class
% with transformations, can only cache a subset of computations here
if ttype > 0
  clikClass = zeros(numtest,numclass+1);
  clikTran  = trandata_indeplike(qq0, 0, pp0, 0, testss, ...
      dtype(1), dparam, ttype, tparam);
else
  clikClass = datastat_indeplike(classqq, classnd, testss, dtype(1), dparam);
end
if usetypeB
  clikClass = clikClass + datastat_indeplike(classqqB, classnd, ...
    testssB, dtype(2), dparamB);
end

% compute posterior probability of each table
tableprob = [tablend tdp.alpha];
tableprob = tableprob/sum(tableprob);
  
% account for table-specific transformations, or combine probabilities
% from all tables corresponding to same class if no transformations
if ttype > 0
  clik = clikClass(:,[tablecc numclass+1]);
  for tt = 1:numtable
    cc    = tablecc(tt);
    ssref = testss - tablerr(:,tt*ones(1,numtest)); % TODO: generic transforms
    clik(:,tt) = clik(:,tt) + datastat_indeplike(classqq(:,cc), ...
      classnd(cc), ssref, dtype(1), dparam);
  end
  clik(:,numtable+1) = clik(:,numtable+1) + clikTran;

  testprob = exp(clik) .* tableprob(ones(numtest,1),:);
  probnorm = sum(testprob,2);
  testprob = testprob ./ probnorm(:,ones(1,numtable+1));
else
  classprob = [zeros(1,numclass) tableprob(end)];
  for tt = 1:numtable
    classprob(tablecc(tt)) = classprob(tablecc(tt)) + tableprob(tt);
  end

  testprob = exp(clikClass) .* classprob(ones(numtest,1),:);
  probnorm = sum(testprob,2);
  testprob = testprob ./ probnorm(:,ones(1,numclass+1));
end

