
function testlik = predictlike(tdp, groups, testdata)
%predictlike     Predictive log-likelihood of data given current assignments
%
%    testlik = predictlike(TDP, groups, testdata)
%
%  Approximates log-likelihood of given data using current
%  class assignments computed by Gibbs sampler, where
%    groups   = groups for which likelihood computed
%    testdata = matrix such that each column is a test data point,
%               or if multiple datatypes a cell array such that
%               newdata{dd} is the data matrix for datatype dd
%    testlik  = [length(groups) x numtestdata] matrix giving log-likelihood
%               of each data point, under each specified group

if ~iscell(testdata)
  testdata = {testdata};
end

% determine number of datatypes used in data likelihoods
% hardcoded for 1 or 2 datatypes to avoid overhead of loop over datatypes
if tdp.numtype > 2
  error('Likelihood evaluation supports at most two data types');
end
usetypeB = (tdp.numtype > 1);

% number of test data points
numtest = size(testdata{1},2);
testlik = zeros(length(groups),numtest);

testss = testdata{1};
if usetypeB
  testssB = testdata{2};
else
  testssB = zeros(0,numtest);
end

% class (part) level state variables
numclass = tdp.numclass;
classnd  = tdp.classnd;
classqq  = tdp.classqq{1};

% data parameters
numtype = tdp.numtype;
dtype   = tdp.dtype;
dparam  = tdp.dparam{1};
qq0     = tdp.qq0{1};

% parameters and state variables for second datatype
if usetypeB
  dparamB  = tdp.dparam{2};
  qq0B     = tdp.qq0{2};
  classqqB = tdp.classqq{2};
end

% transformation parameters
ttype    = tdp.ttype;
tparam   = tdp.tparam;
pp0      = tdp.pp0;

% add empty class to allow evaluation of likelihood for undefined clusters
classnd(numclass+1)   = 0;
classqq(:,numclass+1) = qq0; 
if usetypeB
  classqqB(:,numclass+1) = qq0B;
end

% evaluate log-likelihood of each data point, given each global class
% with transformations, can only cache a subset of computations here
if ttype > 0
  clikClass = zeros(numtest,numclass+1);
  clikTran  = trandata_indeplike(qq0, 0, pp0, 0, testss, ...
      dtype(1), dparam, ttype, tparam);
else
  clikClass = datastat_indeplike(classqq, classnd, testss, dtype(1), dparam);
end
if usetypeB
  clikClass = clikClass + datastat_indeplike(classqqB, classnd, ...
    testssB, dtype(2), dparamB);
end

% average class-specific log-likelihoods for each group
for j = 1:length(groups)
  jj = groups(j);

  % table state variables for specified group
  numtable = tdp.numtable(jj);
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % compute posterior probability of each table
  tableprob = [tablend tdp.alpha];
  tableprob = tableprob/sum(tableprob);
  
  % select class likelihoods for given tables
  clik = clikClass(:,[tablecc numclass+1])';

  % account for table-specific transformations, if needed
  if ttype > 0
    for tt = 1:numtable
      cc    = tablecc(tt);
      ssref = testss - tablerr(:,tt*ones(1,numtest)); % TODO: generic transforms
      clik(tt,:) = clik(tt,:) + datastat_indeplike(classqq(:,cc), ...
        classnd(cc), ssref, dtype(1), dparam)';
    end
    clik(numtable+1,:) = clik(numtable+1,:) + clikTran';
  end

  % average likelihoods according to table probabilities
  maxlike = max(clik);
  maxlike = maxlike(ones(numtable+1,1),:);  % repmat(maxlike,numtable+1,1)
  clik    = exp(clik - maxlike);

  testlik(j,:) = log(tableprob*clik) + maxlike(1,:);
end

