
function tdp = clearassign(tdp, groups, clearobj)
%clearassign    Clear assignments of data to tables for specified groups
%
%    TDP = clearassign(TDP, groups, clearobj)
%
%  Clears internal assignments of data to tables and objects, where
%    groups   = groups for assignments cleared (by default, all)
%    clearobj = flag to determine action for object category labels
%          0 -> clear data assignments, but keep category and instance labels
%          1 -> clear data assignments and category/instance labels (default)
%          2 -> clear category and instance labels, but keep current assignments 
%  Useful to test learning from synthetic data, or to run Gibbs sampling
%  with multiple restarts on test data.

% default parameters
if nargin < 2
  groups = [];
end
if nargin < 3
  clearobj = 1;
end

numgroup = tdp.numgroup;
if isempty(groups)
  groups = [1:numgroup];
end

% object and part state variables
numobj  = tdp.numobj;
objnt   = tdp.objnt;
objpp   = tdp.objpp;
objll   = tdp.objll;
numpart = tdp.numpart;
partnd  = tdp.partnd;
partqq  = tdp.partqq;

% data parameters
dtype   = tdp.dtype;
dparam  = tdp.dparam;
numtype = 2;  

% transformation parameters
ttype   = tdp.ttype;
tparam  = tdp.tparam;

% depth parameters
ztype = tdp.ztype;

% clear assignments for each specified group
for j = 1:length(groups)
  jj = groups(j);

  % table and data level state variables for current group
  numdata  = tdp.numdata(jj);
  numtable = tdp.numtable(jj);
  datass   = tdp.datass{jj}{1};
  dataoo   = tdp.datass{jj}{2};
  datakk   = tdp.datakk{jj};
  datatt   = tdp.datatt{jj};
  datall   = tdp.datall{jj};
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % keep manually labeled table instances along with object labels
  if clearobj == 1
    clearind = find(datatt ~= 0);
    datall(1:numdata) = 0;
  elseif clearobj == 2
    clearind = [];
    datall(1:numdata) = 0;
    datatt = abs(datatt);
  else
    clearind = find(datatt > 0);
  end

  % remove all assigned data items from cached statistics
  for ii = clearind
    oldkk = datakk(ii);
    oldtt = abs(datatt(ii));
    oldcc = tablecc(oldtt);

    if ztype > 0    % TODO: generic transforms
      ssref = [datass(1:2,ii); 1]*datass(3,ii) - tablerr(:,oldtt); 
    else
      ssref = datass(:,ii) - tablerr(:,oldtt);
    end

    partqq{oldcc}{1}(:,oldkk) = datastat_del(partqq{oldcc}{1}(:,oldkk), ...
      partnd{oldcc}(oldkk), ssref, dtype(1), dparam{1});
    partqq{oldcc}{2}(:,oldkk) = datastat_del(partqq{oldcc}{2}(:,oldkk), ...
      partnd{oldcc}(oldkk), dataoo(:,ii), dtype(2), dparam{2});
    partnd{oldcc}(oldkk) = partnd{oldcc}(oldkk) - 1;
    tablend(oldtt) = tablend(oldtt) - 1;
  end
  datakk(clearind) = 0;
  datatt(clearind) = 0;

  % remove deserted tables
  for tt = numtable:-1:1
    if tablend(tt) == 0
      numtable = numtable - 1;
      cc = tablecc(tt);
      objpp(:,cc) = transtat_del(objpp(:,cc), objnt(cc), ...
        tablerr(:,tt), ttype, tparam);
      objnt(cc)   = objnt(cc) - 1;

      ii = find(datatt > tt);
      datatt(ii)    = datatt(ii) - 1;
      tablecc(tt)   = [];
      tablerr(:,tt) = [];
      tablend(tt)   = [];
    end
  end

  % store updated table statistics
  tdp.numtable(jj) = numtable;
  tdp.tablecc{jj}  = tablecc;
  tdp.tablerr{jj}  = tablerr;
  tdp.tablend{jj}  = tablend;
  tdp.datakk{jj}   = datakk;
  tdp.datatt{jj}   = datatt;
  tdp.datall{jj}   = datall;
end

for cc = numobj:-1:1
  % remove any unused parts from each object category
  for kk = numpart(cc):-1:1
    if partnd{cc}(kk) == 0
      numpart(cc) = numpart(cc) - 1;
      partnd{cc}(kk)      = [];
      partqq{cc}{1}(:,kk) = [];
      partqq{cc}{2}(:,kk) = [];

      for jj = 1:numgroup
        tablecc = tdp.tablecc{jj};
        partind = find(tdp.datakk{jj} > kk);
        ii = find(tablecc(abs(tdp.datatt{jj}(partind))) == cc);
        tdp.datakk{jj}(partind(ii)) = tdp.datakk{jj}(partind(ii)) - 1;
      end
    end
  end

  % remove any unused object categories
  if objnt(cc) == 0
    if numpart(cc) > 0
      error('Internal inconsistency between objnt and numpart');
    end

    numobj      = numobj - 1;
    objnt(cc)   = [];
    objpp(:,cc) = [];
    objll(cc)   = [];
    numpart(cc) = [];
    partnd(cc)  = [];
    partqq(cc)  = [];

    for jj = 1:numgroup
      tt = find(tdp.tablecc{jj} > cc);
      tdp.tablecc{jj}(tt) = tdp.tablecc{jj}(tt) - 1;
    end
  end
end

% rebuild global object labels, in case features which induced a label removed
newll = zeros(size(objll));
for jj = 1:numgroup
  tablecc  = tdp.tablecc{jj};
  datatt   = abs(tdp.datatt{jj});
  datall   = tdp.datall{jj};
  labelind = find((datatt > 0) & (datall ~= 0));

  for ii = labelind
    cc = tablecc(datatt(ii));       

    if datall(ii) > 0
      newll(cc) = datall(ii);
    elseif newll(cc) == 0
      newll(cc) = datall(ii);
    end
  end
end

% store updated object and part statistics
tdp.numobj  = numobj;
tdp.objnt   = objnt;
tdp.objpp   = objpp;
tdp.objll   = newll;
tdp.numpart = numpart;
tdp.partnd  = partnd;
tdp.partqq  = partqq;

