
function tdp = insertdata(tdp, newdata, group, labels, instance, method)
%insertdata     Add new data items to the specified group
%
%    TDP = insertdata(TDP, newdata, group, labels, instance, method)
%
%  Adds new data items to specified group, where
%    newdata  = matrix such that each column is a different data point,
%               or if multiple datatypes a cell array such that
%               newdata{dd} is the data matrix for datatype dd
%    group    = single group in which to add data (existing data
%               in that group is appended to, not replaced)
%    labels   = optional labels specifying object category for these
%               features, or single label for all features, where 
%               negative labels denote unallowed assignments, and
%               0 denotes unlabeled features (default)
%    instance = optional positive integers encoding individual object
%               instances, assumed distinct from instances in previous
%               calls to this function, or 0 for no instance label (default)
%    method   = scheme used to assign instance tables to visual categories:
%               0 -> sample visual category given previous instances (default)
%               1 -> same visual category for all instances of each object
%               2 -> different visual category for all instances of each object

%Sample parameters for 3D data:
%newdata{1} = 3xM matrix for M features, where newdata{1}(1:2,:) gives feature
%             position in normalized image coordinates and newdata{1}(3,:)
%             gives starting depth in normalized world coordinates (vx,vy,uz)
%newdata{2} = 1xM vector giving discrete appearance of each feature
%newdata{3} = ZxM matrix for M features, where each column gives a vector
%             of depth log-likelihoods evaluated at dataprior{3}.grid

if nargin < 3
  group = 1;
end
if nargin < 4
  labels = 0;
end
if nargin < 5
  instance = 0;
end
if nargin < 6
  method = 0;
end

% number of auxiliary variable sampling iterations for sequential assignments
numaux = 5;

% static parameters
gamma   = tdp.gamma;
conpart = tdp.conpart;

% feature position/appearance parameters
dtype  = tdp.dtype;
dparam = tdp.dparam;
qq0    = tdp.qq0;

% transformation parameters
ttype   = tdp.ttype;
tparam  = tdp.tparam;
pp0     = tdp.pp0;

% depth parameters
ztype = tdp.ztype;

% object state variables
numobj  = tdp.numobj;
objnt   = tdp.objnt;
objpp   = tdp.objpp;
objll   = tdp.objll;
numpart = tdp.numpart;

% part level state variables
partnd = tdp.partnd;
partqq = tdp.partqq;

% table level state variables for specified group
numtable = tdp.numtable(group);
tablecc  = tdp.tablecc{group};
tablerr  = tdp.tablerr{group};
tablend  = tdp.tablend{group};

% check that newdata has proper number of types and is consistent
if ~iscell(newdata)
  newdata = {newdata};
end
if length(newdata) ~= tdp.numtype
  error('Wrong number of types in newdata');
end
numdata = size(newdata{1},2);
for dd = 2:tdp.numtype
  if size(newdata{dd},2) ~= numdata
    error('Internal inconsistency in newdata');
  end
end

% replicate labels if needed
if length(labels) == 1
  labels = labels*ones(1,numdata);
end
if length(instance) == 1
  instance = instance*ones(1,numdata);
end

% add empty part to each object category
for cc = 1:numobj
  partnd{cc} = [partnd{cc}, 0];
  for dd=1:2
    partqq{cc}{dd} = [partqq{cc}{dd}, qq0{dd}];
  end
end

% add empty visual object category 
objnt(numobj+1)   = 0;
objpp(:,numobj+1) = pp0;
objll(numobj+1)   = 0;
numpart(numobj+1) = 0;
partnd{numobj+1}  = 0;
for dd=1:2
  partqq{numobj+1}{dd} = qq0{dd};
end

% Account for object instance labels by assigning to tables
newdatatt = zeros(1, numdata);
newdatakk = zeros(1, numdata);
names     = unique(instance);
names     = names(find(names > 0));

for aa = names
  ttind = find(instance == aa);
  ss    = newdata{1}(:,ttind);
  oo    = newdata{2}(:,ttind);

  if ztype > 0
    ss(1:2,:) = ss(1:2,:) .* ss([3;3],:); % project to 3D
  end

  % check consistency of object category labels for this instance
  newll = unique(labels(ttind));
  if length(newll) > 1
    error('All features assigned to given instance must have same category');
  end

  % determine object category to which this table is assigned
  if method == 0
    if newll > 0
      validcc = find((objll == newll) | (objll == 0) | ...
                    ((objll ~=-newll) & (objll < 0)));
    elseif newll < 0 
      validcc = find(objll ~= -newll);
    else
      validcc = [1:numobj+1];
    end

    clikcc = zeros(1,length(validcc));
    refcc  = zeros(size(ss,1),length(validcc)); 
    for c = 1:length(validcc)
      cc = validcc(c);
      [clikcc(c),refcc(:,c)] = gibbs_llikTable(numaux, ss, oo, conpart, ...
        partnd{cc}, partqq{cc}{1}, partqq{cc}{2}, objpp(:,cc), objnt(cc), ...
        dtype(1), dparam{1}, dtype(2), dparam{2}, ttype, tparam);
    end

    clikcc = clikcc/size(ss,2);  % heuristic annealing of likelihoods
    clik   = exp(clikcc-max(clikcc)) .* [objnt(validcc(1:end-1)), gamma];
    newcc  = multinomial_rnd(clik, 1);
    newrr  = refcc(:,newcc);
    newcc  = validcc(newcc);

  elseif method == 1
    newcc = find(objll == newll);
    if length(newcc) > 1
      warning('Multiple global categories, arbitrarily assigning instance to first');
      newcc = newcc(1);
    elseif length(newcc) == 0
      newcc = numobj+1;
    end

    [clikdummy,newrr] = gibbs_llikTable(numaux, ss, oo, conpart, ...
      partnd{newcc}, partqq{newcc}{1}, partqq{newcc}{2}, objpp(:,newcc), ...
      objnt(newcc), dtype(1), dparam{1}, dtype(2), dparam{2}, ttype, tparam);
  else
    newcc = numobj+1;
    newrr = zeros(size(ss,1),1); %TODO: generic transforms
  end

  % add new visual object category if needed
  if newcc > numobj
    numobj = newcc;

    objnt(numobj+1)   = 0;
    objpp(:,numobj+1) = pp0;
    objll(numobj+1)   = 0;
    numpart(numobj+1) = 0;
    partnd{numobj+1}  = 0;
    for dd=1:2
      partqq{numobj+1}{dd} = qq0{dd};
    end
  end

  % sample new assignments of features to parts
  ssref = ss - newrr(:,ones(1,length(ttind))); % TODO: generic transforms
  for ii = randperm(length(ttind))
    % likelihood and prior probability of each part
    clik   = datastat_condlike(partqq{newcc}{1}, partnd{newcc}, ...
      ssref(:,ii), dtype(1), dparam{1});
    clik   = clik + datastat_condlike(partqq{newcc}{2}, partnd{newcc}, ...
      oo(:,ii), dtype(2), dparam{2});
    probkk = [partnd{newcc}(1:numpart(newcc)), conpart];

    % sample assignment according to these probabilities
    newkk = multinomial_rnd(exp(clik-max(clik)).*probkk, 1);

    % add a new part if needed
    if newkk > numpart(newcc)
      numpart(newcc) = newkk;

      partnd{newcc} = [partnd{newcc}, 0];
      for dd = 1:2
        partqq{newcc}{dd} = [partqq{newcc}{dd}, qq0{dd}];
      end
    end

    % add feature to statistics of chosen part
    partqq{newcc}{1}(:,newkk) = datastat_add(partqq{newcc}{1}(:,newkk), ...
      partnd{newcc}(newkk), ssref(:,ii), dtype(1), dparam{1});
    partqq{newcc}{2}(:,newkk) = datastat_add(partqq{newcc}{2}(:,newkk), ...
      partnd{newcc}(newkk), oo(:,ii), dtype(2), dparam{2});
    partnd{newcc}(newkk)      = partnd{newcc}(newkk) + 1;

    newdatakk(ttind(ii)) = newkk; 
  end

  % update table statistics 
  newtt    = numtable+1;
  numtable = newtt;

  tablend(newtt)   = length(ttind);
  tablecc(newtt)   = newcc;
  tablerr(:,newtt) = newrr;
  objpp(:,newcc)   = transtat_add(objpp(:,newcc), objnt(newcc), newrr, ttype, tparam);
  objnt(newcc)     = objnt(newcc) + 1;

  % use negative numbers to indicate fixed assignments of data to table
  newdatatt(ttind)  = -newtt;

  % update object label for this category, if necessary
  if newll > 0
    objll(newcc) = newll;
  elseif (newll < 0) && (objll(newcc) == 0)
    objll(newcc) = newll;
  end
end

% add newdata to specified group
tdp.numdata(group) = tdp.numdata(group) + numdata;
tdp.datakk{group}  = [tdp.datakk{group}, newdatakk];
tdp.datatt{group}  = [tdp.datatt{group}, newdatatt];
for dd = 1:tdp.numtype
  tdp.datass{group}{dd} = [tdp.datass{group}{dd}, newdata{dd}];
end
tdp.datall{group}  = [tdp.datall{group}, labels];

% remove empty dummy object category 
objnt(numobj+1)   = [];
objpp(:,numobj+1) = [];
objll(numobj+1)   = [];
numpart(numobj+1) = [];
partnd(numobj+1)  = [];
partqq(numobj+1)  = [];

% remove empty parts from object categories
for cc = 1:numobj
  partnd{cc}(numpart(cc)+1) = [];
  for dd=1:2
    partqq{cc}{dd}(:,numpart(cc)+1) = [];
  end
end

% store updated table statistics
tdp.numtable(group) = numtable;
tdp.tablecc{group}  = tablecc;
tdp.tablerr{group}  = tablerr;
tdp.tablend{group}  = tablend;

% store updated object and part statistics
tdp.numobj  = numobj;
tdp.objnt   = objnt;
tdp.objpp   = objpp;
tdp.objll   = objll;
tdp.numpart = numpart;
tdp.partnd  = partnd;
tdp.partqq  = partqq;

