
function tdp = sampledata(tdp, groups, numsamp, maxobj, maxtable, maxpart)
%sampledata     Generate random datasets using specified, or random, parameters
%
%    TDP = sampledata(TDP, groups, numsamp, maxobj, maxtable, maxpart)
%
%  Generate synthetic dataset by random sampling, where
%    groups   = groups for which synthetic data created (all, by default)
%    numsamp  = number of new data items sampled for each group
%    maxobj   = maximum number of object categories instantiated by
%               sampled data (zero for no limit, the default)
%    maxtable = maximum number of transformed objects (tables) created
%               within each group (zero for no limit, the default)
%    maxpart  = maximum number of parts created for each object category
%               (zero for no limit, the default)

% Default parameters
if nargin < 2
  groups = [];
end
if nargin < 3
  numsamp = 100;
end
if nargin < 4
  maxobj = 0;
end
if nargin < 5
  maxtable = 0;
end
if nargin < 6
  maxpart = 0;
end

if isempty(groups)
  groups = [1:tdp.numgroup];
end

% static parameters
gamma    = tdp.gamma;
alpha    = tdp.alpha;
conpart  = tdp.conpart;
numgroup = tdp.numgroup;
numtype  = 2;   % TODO: adapt to 3D data?

% feature position/appearance parameters
dtype  = tdp.dtype;
dparam = tdp.dparam;
qq0    = tdp.qq0;

% transformation parameters
ttype   = tdp.ttype;
tparam  = tdp.tparam;
pp0     = tdp.pp0;

% object state variables
numobj  = tdp.numobj;
objnt   = tdp.objnt;
objpp   = tdp.objpp;
objll   = tdp.objll;
numpart = tdp.numpart;

% part level state variables
partnd = tdp.partnd;
partqq = tdp.partqq;

% determine parameters associated with existing objects
thetapp = transtat_estparam(objpp, objnt, ttype, tparam);
for cc = 1:numobj
  for dd = 1:numtype
    thetaqq{cc}{dd} = datastat_estparam(partqq{cc}{dd}, ...
      partnd{cc}, dtype(dd), dparam{dd});
  end
end

% Sample dataset for each specified group
for j = 1:length(groups)
  jj = groups(j);

  % table and data level state variables for current group
  numtable = tdp.numtable(jj);
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  % Independently sample each data item
  newdatass = [];
  newdataoo = [];
  newdatakk = zeros(1,numsamp);
  newdatatt = zeros(1,numsamp);
  newdatall = zeros(1,numsamp);

  for ii = 1:numsamp
    % sample table associated with data item
    if (maxtable > 0) && (numtable >= maxtable)
      newtt = multinomial_rnd(tablend, 1);
    else
      newtt = multinomial_rnd([tablend alpha], 1);
    end

    % add a new table if needed
    if newtt > numtable
      % sample object associated with new table
      if (maxobj > 0) && (numobj >= maxobj)
        newcc = multinomial_rnd(objnt, 1);
      else
        if isinf(gamma)
          newcc = numobj + 1;
        else 
          newcc = multinomial_rnd([objnt gamma], 1);
        end
      end

      % add a new object if needed
      if newcc > numobj
        numobj         = newcc;
        objnt(newcc)   = 0;
        numpart(newcc) = 0;
        objll(newcc)   = newcc;

        objpp(:,newcc)   = pp0;
        thetapp(:,newcc) = transtat_sampparam(pp0, 0, 1, ttype, tparam);

        partnd{newcc} = [];
        for dd = 1:numtype
          partqq{newcc}{dd}  = [];
          thetaqq{newcc}{dd} = [];
        end
      end

      % sample transformation for new table
      tablerr(:,newtt) = tranparam_samptran(thetapp(:,newcc), 1, ttype, tparam);
      objpp(:,newcc)   = transtat_add(objpp(:,newcc), objnt(newcc), ...
        tablerr(:,newtt), ttype, tparam);

      % update statistics to account for new table
      numtable       = newtt;
      tablend(newtt) = 0;
      tablecc(newtt) = newcc;
      objnt(newcc)   = objnt(newcc) + 1;
    else
      newcc = tablecc(newtt); 
    end

    % sample part associated with data item
    if (maxpart > 0) && (numpart(newcc) >= maxpart)
      newkk = multinomial_rnd(partnd{newcc}, 1);
    else
      newkk = multinomial_rnd([partnd{newcc} conpart], 1);
    end

    % add a new part if needed
    if newkk > numpart(newcc)
      numpart(newcc) = newkk;

      partnd{newcc}(newkk) = 0;
      for dd = 1:numtype
        partqq{newcc}{dd}(:,newkk)  = qq0{dd};
        thetaqq{newcc}{dd}(:,newkk) = datastat_sampparam(qq0{dd}, 0, 1, ...
          dtype(dd), dparam{dd});
      end
    end

    % sample data item using object/part parameters, and update statistics
    newdatass(:,ii) = dataparam_sampdata(thetaqq{newcc}{1}(:,newkk), ...
      1, dtype(1), dparam{1});
    partqq{newcc}{1}(:,newkk) = datastat_add(partqq{newcc}{1}(:,newkk), ...
      partnd{newcc}(newkk), newdatass(:,ii), dtype(1), dparam{1});
    % TODO: generic transformations
    newdatass(:,ii)  = newdatass(:,ii) + tablerr(:,newtt);

    newdataoo(:,ii) = dataparam_sampdata(thetaqq{newcc}{2}(:,newkk), ...
      1, dtype(2), dparam{2});
    partqq{newcc}{2}(:,newkk) = datastat_add(partqq{newcc}{2}(:,newkk), ...
      partnd{newcc}(newkk), newdataoo(:,ii), dtype(2), dparam{2});
    partnd{newcc}(newkk) = partnd{newcc}(newkk) + 1;

    newdatall(ii)  = newcc;
    newdatakk(ii)  = newkk;
    newdatatt(ii)  = newtt;
    tablend(newtt) = tablend(newtt) + 1;
  end

  % store data, updated tables, and generating table assignments
  tdp.numdata(jj)   = tdp.numdata(jj) + numsamp;
  tdp.numtable(jj)  = numtable;
  tdp.tablecc{jj}   = tablecc;
  tdp.tablerr{jj}   = tablerr;
  tdp.tablend{jj}   = tablend;

  tdp.datakk{jj}    = [tdp.datakk{jj}, newdatakk];
  tdp.datatt{jj}    = [tdp.datatt{jj}, newdatatt];
  tdp.datall{jj}    = [tdp.datall{jj}, newdatall];
  tdp.datass{jj}{1} = [tdp.datass{jj}{1}, newdatass];
  tdp.datass{jj}{2} = [tdp.datass{jj}{2}, newdataoo];
end

% store updated object and part statistics
tdp.numobj  = numobj;
tdp.objnt   = objnt;
tdp.objpp   = objpp;
tdp.objll  = objll;
tdp.numpart = numpart;
tdp.partnd  = partnd;
tdp.partqq  = partqq;

