        
function visualize(tdp, groups, plotstyle, Hfig)
%visualize     Visualize posterior distributions implied by current assignments
%
%    visualize(TDP, groups, plotstyle, Hfig)
%
%  Visualize classes implied by current assignments, where 
%    groups    = groups to visualize transformed classes, or tables (max five)
%    plotstyle = struct of plot parameters
%    Hfig      = figure handle (if empty, a new figure is created)

  if nargin < 2
    groups = [];
  end
  if nargin < 3
    plotstyle = struct([]);
  end
  if nargin < 4
    Hfig = [];
  end
  
  % feature position and transformation parameters
  dtype  = tdp.dtype(1);
  dparam = tdp.dparam{1};
  ttype  = tdp.ttype;
  
  % object state variables
  numobj  = tdp.numobj;
  objnt   = tdp.objnt;
  numpart = tdp.numpart;

  % part level state variables
  partnd = tdp.partnd;
  partqq = tdp.partqq;

  % create figure, if needed
  if isempty(Hfig)
    Hfig = figure;
  end
  
  % preprocess variables needed to plot tables for each group
  numplot = length(groups);
  if numplot > 5
    warning('Only displaying tables for first five groups');
    groups = groups(1:5);
    numplot = 5;
  end
  numrows = ceil((numplot+1)/2);
  
  % posterior position parameters for all parts of each object
  for cc = 1:numobj
    thetaqq = datastat_estparam(partqq{cc}{1}, partnd{cc}, dtype, dparam);
    dim     = dparam(1);
    mu{cc}  = thetaqq(1:dim,:);

    if dtype == 2
      sigma{cc} = reshape(thetaqq(dim+1:end,:),dim,dim,numpart(cc));
    else
      sigma{cc} = zeros(dim,dim,numpart(cc));
      for kk = 1:numpart(cc)
        sigma{cc}(:,:,kk) = diag(thetaqq(dim+1:end,kk));
      end
    end
  end
  
  % plot global parts
  if numplot > 0
    Hax = subplot(numrows,2,1,'Parent',Hfig);
  else
    Hax = subplot(1,1,1,'Parent',Hfig);
  end

  if isfield(plotstyle, 'colors')
    colors = plotstyle.colors;
    colors = repmat(colors,1,ceil(numobj/length(colors)));
  else
    colors = repmat({'b'},1,numobj);
  end

  % realign mean values to account for empirical mean of current tables
  muTables = zeros(dim,numobj);
  for jj = 1:tdp.numgroup
    tablecc  = tdp.tablecc{jj};
    tablerr  = tdp.tablerr{jj};
    numtable = tdp.numtable(jj);

    for tt = 1:numtable
      muTables(:,tablecc(tt)) = muTables(:,tablecc(tt)) + tablerr(:,tt);
    end
  end
  muTables = muTables ./ objnt(ones(dim,1),:);

  cla(Hax);
  hold(Hax,'on');
  for cc = 1:numobj
    plotGaussMix(mu{cc} + muTables(:,cc*ones(1,numpart(cc))), ...
      sigma{cc}, partnd{cc}/sum(partnd{cc}), colors{cc}, Hax);
  end
  if isfield(plotstyle, 'range')
    axis(Hax, plotstyle.range);
  end

  for j = 1:numplot
    jj = groups(j);

    tablecc  = tdp.tablecc{jj};
    tablerr  = tdp.tablerr{jj};
    tablend  = tdp.tablend{jj};
    numtable = tdp.numtable(jj);

    muT = []; sigmaT = []; weightsT = []; colorsT = {};
    for tt = 1:numtable
      cc = tablecc(tt);

      muT      = cat(2, muT, mu{cc}+tablerr(:,tt*ones(1,numpart(cc))));
      sigmaT   = cat(3, sigmaT, sigma{cc});
      weightsT = cat(2, weightsT, tablend(tt)*(partnd{cc}/sum(partnd{cc})));

      colorsT = cat(2, colorsT, repmat(colors(cc),1,numpart(cc)));
    end

    Hax = subplot(numrows,2,j+1,'Parent',Hfig);
    cla(Hax);
    plotGaussMix(muT, sigmaT, weightsT, colorsT, Hax);
    if isfield(plotstyle, 'range')
      axis(Hax, plotstyle.range);
    end
  end

