
% Script for experiments on parametric LDA object-part scene model (context)

savename = 'sceneLDA';  % path and filename prefix for results

useStreet = 1;

datapath = '../database';

if useStreet
  load(fullfile(datapath, 'streetFixScale/streetFixScale_2006_mser_ha_edges.mat'));
  baddocs  = [37 52 54 107 114 117 124 129 131 136 138 140 165 177 257 309 325 434 444 466 483 503 510 601];
  newlabel = -1;   % object category label/restriction for "unlabeled" training
  numobj   = 4;
  
  pruneSmallFeatures = true;
  sampref = true;
else
  load(fullfile(datapath, 'office/office_monocular_2006_mser_ha_edges.mat'));
  baddocs  = [1 159 285 287 318];
  newlabel = 4;
  numobj   = 4;

  pruneSmallFeatures = false;
  sampref = true;
end

rand('state',1); randn('state',1);

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

% Clustering hyperparameters
numclass = 8*numobj;  % number of shared object parts
alpha0   = 15;        % base clustering rate, divided equally among parts
gamma    = 1;         % scene clustering rate

% Flag to use detected feature positions, or appearance only
usePosition  = 1;
useReference = 1;

% Training parameters
numtrain      = 250;  % number of scene training images
numgibbsTrain = 500;  % number of training Gibbs sampling iterations
numEM         = 0;    % number of reference EM updates per Gibbs sample

% Testing parameters
numtest       = inf;  % number of scene test images (inf uses all except training)
numgibbsTest  = 20;   % number of test Gibbs sampling iterations (reference only)
numtrialTest  = 10;   % number of test Gibbs restarts (reference only)

% Gaussian hyperparameters and visualization (less important)
dataprior{1}.type   = 'gaussian';
dataprior{1}.emean  = [128; 128];
dataprior{1}.cmean  = 0.1;
dataprior{1}.ecovar = (25^2)*eye(2); % square of diameter of the part
dataprior{1}.ccovar = 6;

% Discrete hyperparameters and visualization (less important)
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = imagedata.dictionary.numwords; 
dataprior{2}.dirweight  = 0.1;    % Beta in fig 1

% Reference hyperparameters
refprior.emean  = repmat([128; 128],numobj,1);
refprior.cmean  = 0.1;
refprior.ecovar = 5000*eye(2*numobj);
refprior.ccovar = 6;


%%%%%%%%%%%
%% Learning
%%%%%%%%%%%

if ~usePosition
  dataprior = dataprior(2);
end
if ~useReference
  refprior = [];
end

% Remove any flagged bad images
gooddocs = setdiff(1:length(imagedata.files),baddocs);
imagedata.imgsize         = imagedata.imgsize(gooddocs);
imagedata.descriptor_left = imagedata.descriptor_left(gooddocs);
imagedata.appearance      = imagedata.appearance(gooddocs);
imagedata.shapeROI_left   = imagedata.shapeROI_left(gooddocs);
imagedata.typeROI_left    = imagedata.typeROI_left(gooddocs);
imagedata.overlap         = imagedata.overlap(gooddocs);
imagedata.classes         = imagedata.classes(gooddocs);
imagedata.files           = imagedata.files(gooddocs);
imagedata.instance        = imagedata.instance(gooddocs);

% Initialize scene hierarchical model
numgroup = length(imagedata.files);
SO = sceneobject(numgroup,1,numobj,numclass,alpha0/numclass,gamma,dataprior,refprior);

% Prune small features
if pruneSmallFeatures
  for jj = 1:numgroup
    t = imagedata.typeROI_left{jj}';
    [orientation,s,aspectRatio,x,y] = getROIproperties(imagedata.shapeROI_left{jj});
    ii = find(((s>10 & t==1) | (s>15 & t==2) | (s>25 & t ==3)) & (s<220));
    
    % remove data from struct
    imagedata.descriptor_left{jj} = imagedata.descriptor_left{jj}(:,ii);
    imagedata.appearance{jj}      = imagedata.appearance{jj}(ii);
    imagedata.shapeROI_left{jj}   = imagedata.shapeROI_left{jj}(:,ii);
    imagedata.typeROI_left{jj}    = imagedata.typeROI_left{jj}(ii);
    imagedata.overlap{jj}         = imagedata.overlap{jj}(:,ii);
    imagedata.classes{jj}         = imagedata.classes{jj}(ii);
    imagedata.instance{jj}        = imagedata.instance{jj}(ii);
    
    disp(sprintf('%04d: %d --> %d', jj, length(t), length(ii)));
  end
end

% Select training images
perm = randperm(numgroup);
traingroup = sort(perm(1:numtrain));
testgroup  = setdiff([1:numgroup], traingroup);
testgroup  = testgroup(1:min(numtest,length(testgroup)));

% Insert data into model
fprintf(1, 'Initializing scene model...\n');
for jj = 1:numgroup 
  if usePosition
    x = imagedata.shapeROI_left{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
    y = imagedata.shapeROI_left{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
    newdata{1} = [x; y];
    newdata{2} = imagedata.appearance{jj};
  else
    newdata{1} = imagedata.appearance{jj};
  end

  if find(traingroup == jj)
    objlabel = imagedata.classes{jj};
    objlabel(find(objlabel==0)) = newlabel;
    SO = insertdata(SO, newdata, jj, 1, objlabel);
  else
    SO = insertdata(SO, newdata, jj, 1, 0);
  end
end

% Gibbs sampling
fprintf(1, 'Training with %d examples...\n', numtrain); 
for i = 1:10
  SO = doGibbs(SO, numgibbsTrain/10, [], traingroup, [], numEM);
  save(sprintf('%s_train%02d',savename,i));
end

% Assignment probabilities for later performance analysis
if useReference
  fprintf(1, 'Test image Gibbs sampling for reference positions...\n');
  objprob   =  cell(numtrialTest,length(testgroup));
  sceneprob = zeros(numtrialTest,length(testgroup));
  for i = 1:numtrialTest
    SO = clearlabels(SO, testgroup, true, sampref);
    SO = doGibbs(SO, numgibbsTest, [], testgroup, [], numEM);

    sceneprob(i,:) = estcategory(SO, testgroup);
    for j = 1:length(testgroup)
      objprob{i,j} = estassign(SO, testgroup(j));
    end
    save(sprintf('%s_test%02d',savename,i));
  end
else
  sceneprob = estcategory(SO, testgroup);
  objprob   = cell(1,length(testgroup));
  for j = 1:length(testgroup)
    objprob{j} = estassign(SO, testgroup(j));
  end
  save(sprintf('%s_test',savename));
end

