
% Script for TDP experiments on multiple object scenes (with parts)

savename = '';  % path and filename prefix for results

useStreet = 1;
showPlots = 1;

datapath = '../database';

if useStreet
  load(fullfile(datapath, 'streetFixScale/streetFixScale_2006_mser_ha_edges.mat'));
  baddocs    = [37 52 54 107 114 117 124 129 131 136 138 140 165 177 257 309 325 434 444 466 483 503 510 601];
  newlabel   = -1;    % object category label/restriction for "unlabeled" training
  noinstance = [2:4]; % remove instance labels for textural categories

  imagepath  = fullfile(datapath, 'streetFixScale/Images');
  plotorder  = [0 2 3 4 1];
  plotcolor  = [0.5 0.5 0.5; 1 0 0; 1 0 1; 0 0 1; 0 1 0];
  plotstyle  = {'k', 'r', 'm', 'b', 'g'};
  plotdetail = 1;
  plottable  = [1:4];
  scenerange = [0 342 0 256];
  %showImages = [1 2 4 5 6 7 9 10 11 12 13 14 16 18];

  pruneSmallFeatures = true;
else
  load(fullfile(datapath, 'office/office_monocular_2006_mser_ha_edges.mat'));
  baddocs    = [1 159 285 287 318];
  newlabel   = 4;     % object category label/restriction for "unlabeled" training
  noinstance = [];    % remove instance labels for textural categories

  imagepath  = fullfile(datapath, 'office/Images');
  plotorder  = [0 4 1 3 2];
  plotcolor  = [0.5 0.5 0.5; 1 0 0; 0 0 1; 0 1 0; 0.5 0.5 0.5];
  plotstyle  = {'k', 'r', 'b', 'g', 'k'};
  plotdetail = 1;
  plottable  = [1:3];
  scenerange = [0 256 0 256];
  %showImages = [1 3 5 6 7 10 11 12 13 14 15 21 22 28];

  pruneSmallFeatures = false;
end

rand('state',1); randn('state',1);

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

% Clustering hyperparameters
alpha    = [1  1];   % DP prior on object instances (tables) per image
gamma    = [1 .1];   % DP prior on visual categories
nu       = [1 .1];   % DP prior on category-specific parts
%nu      = 0;        % nu=0 constrains each visual category to a single part

% Training parameters
numtrain   = 250;     % number of scene training images
numauxiter = 5;       % auxiliary iterations (table reassignment) per Gibbs iteration
numconiter = 50;      % concentration parameter iterations per Gibbs iteration (fast)
numgibbsTrain = [500 100];    % numgibbsTrain(aa) iterations on the categories with
traincat = {[1:4],[]};        %   labels traincat{aa} (empty specifies all)

% Optional annealing schedule (probably ineffective)
anneal = zeros(1,max(numgibbsTrain)); 

% Testing parameters
numtest       = inf;  % number of scene test images (inf uses all except training)
numtrialTest  = 10;   % number of random initializations used for test images
numsampTest   = 10;   % number of Gibbs samples to store per trial
numspaceTest  = 5;    % number of Gibbs test iterations between each stored sample

% Gaussian hyperparameters and visualization (less important)
%dataprior{1}.type   = 'gaussian';
dataprior{1}.type   = 'diagonalgaussian';
dataprior{1}.emean  = [128; 128];
dataprior{1}.cmean  = 0.1;
dataprior{1}.ecovar = (25^2)*eye(2); % square of diameter of the part
dataprior{1}.ccovar = 6;

% Discrete hyperparameters and visualization (less important)
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = imagedata.dictionary.numwords; 
dataprior{2}.dirweight  = 0.1;    

% Reference hyperparameters
%tranprior.type   = 'zeromeangaussian';
tranprior.type   = 'zeromeandiagonalgaussian';
tranprior.ecovar = 5000*eye(2);
tranprior.ccovar = 6;

usePosition = 1;

%%%%%%%%%%%
%% Learning
%%%%%%%%%%%

% Remove any flagged bad images
gooddocs = setdiff(1:length(imagedata.files),baddocs);
imagedata.imgsize         = imagedata.imgsize(gooddocs);
imagedata.descriptor_left = imagedata.descriptor_left(gooddocs);
imagedata.appearance      = imagedata.appearance(gooddocs);
imagedata.shapeROI_left   = imagedata.shapeROI_left(gooddocs);
imagedata.typeROI_left    = imagedata.typeROI_left(gooddocs);
imagedata.overlap         = imagedata.overlap(gooddocs);
imagedata.classes         = imagedata.classes(gooddocs);
imagedata.files           = imagedata.files(gooddocs);
imagedata.instance        = imagedata.instance(gooddocs);

% Initialize scene hierarchical model
numgroup = length(imagedata.files);
TDP = tdpscenepart(numgroup,alpha,gamma,nu,dataprior,tranprior);

% Replace non-labeled class, remove specified instance labels, prune small features
for jj = 1:numgroup
  classes  = imagedata.classes{jj};
  instance = imagedata.instance{jj};

  for aa = noinstance
    instance(classes==aa) = 0;
  end
  classes(classes==0) = newlabel;

  imagedata.classes{jj}  = classes;
  imagedata.instance{jj} = instance;

  if pruneSmallFeatures
    t = imagedata.typeROI_left{jj}';
    [orientation,s,aspectRatio,x,y] = getROIproperties(imagedata.shapeROI_left{jj});
    ii = find(((s>10 & t==1) | (s>15 & t==2) | (s>25 & t ==3)) & (s<220));
    
    % remove data from struct
    imagedata.descriptor_left{jj} = imagedata.descriptor_left{jj}(:,ii);
    imagedata.appearance{jj}      = imagedata.appearance{jj}(ii);
    imagedata.shapeROI_left{jj}   = imagedata.shapeROI_left{jj}(:,ii);
    imagedata.typeROI_left{jj}    = imagedata.typeROI_left{jj}(ii);
    imagedata.overlap{jj}         = imagedata.overlap{jj}(:,ii);
    imagedata.classes{jj}         = imagedata.classes{jj}(ii);
    imagedata.instance{jj}        = imagedata.instance{jj}(ii);
    
    disp(sprintf('%04d: %d --> %d', jj, length(t), length(ii)));
  end
end

% Select training images
perm = randperm(numgroup);
traingroup = sort(perm(1:numtrain));
testgroup  = setdiff([1:numgroup], traingroup);
testgroup  = testgroup(1:min(numtest,length(testgroup)));

% Insert data into model
fprintf(1, 'Initializing scene model...\n');
for jj = 1:numgroup
  if usePosition
    x = imagedata.shapeROI_left{jj}(1,:) + 1; % add 1 for C-->Matlab coordinates
    y = imagedata.shapeROI_left{jj}(2,:) + 1; % add 1 for C-->Matlab coordinates
    newdata{1} = [x; y];
    newdata{2} = imagedata.appearance{jj};
  else
    newdata{1} = imagedata.appearance{jj};
  end

  if find(traingroup == jj)
    catlabel = imagedata.classes{jj};
    instance = imagedata.instance{jj};
    TDP = insertdata(TDP, newdata, jj, catlabel, instance);
  else
    TDP = insertdata(TDP, newdata, jj, 0, 0);
  end
end

% Training
if showPlots
  Hglob = figure('name', 'Global Object Clusters');
  Hseg  = figure('name', 'Training Image Segmentations');
  plotgroup = traingroup([1:min(numtrain,7)]);
end

fprintf(1, 'Training with %d examples...\n', numtrain); 
for aa = 1:length(numgibbsTrain)
  for iter = 1:numgibbsTrain(aa)
    TDP = doGibbs(TDP, 1, numauxiter, {}, traingroup, numconiter, ...
      traincat{aa}, false, anneal(iter));
  
    if showPlots
      scenepartTDP_visglob(TDP, plotstyle, 1.0, 1.0, scenerange, Hglob);
      scenepartTDP_visseg(TDP, plotgroup, [], imagedata, imagepath, ...
        plotorder, plotcolor, plotstyle, plotdetail, plottable, Hseg);
      drawnow;
    end
  
    if (~isempty(savename)) && (mod(iter,numgibbsTrain(aa)/10) == 0)
      save(sprintf('%s_train%d_%02d',savename,aa,iter*10/numgibbsTrain(aa)));
    end
  end
end

% Testing
testProb = cell(numtrialTest,numsampTest);

if showPlots
  Htest  = figure('name', 'Test Image Segmentations');
end

for aa = 1:numtrialTest
  for bb = 1:numsampTest
    testProb{aa,bb} = cell(1,numgroup);
  end

  % sample each test group independently, to avoid erroneous generalization
  for jj = testgroup
    TDPtest = TDP;

    for bb = 1:numsampTest
      TDPtest = doGibbs(TDPtest, numspaceTest, numauxiter, {}, jj, 0, [], true);
      testProb{aa,bb}(jj) = estassign(TDPtest, jj, 0);
    end

    if showPlots
      scenepartTDP_visseg(TDPtest, jj, testProb(1:aa,:), imagedata, imagepath, ...
        plotorder, plotcolor, plotstyle, plotdetail, plottable, Htest);
      drawnow;
    end
  end

  if (~isempty(savename))
    save(sprintf('%s_test%02d',savename,aa), 'testProb');
  end
end

