
% Object-part parametric (LDA/author-topic) models for object categories
% Visualization of detection/recognition performance

% load raw data
load sharing_ijcv16;    

% load result file(s)
result{1} = load('app2a_prob12');
result{2} = load('app2b_prob30');

% plot parameters
axisROC  = [0 0.5 0 1];
axisArea = [0.5 1 0.5 1];
plotArea = 1;   % plot individual category ROC, or ROC area scatter plot

% combine variables storing simulation results from multiple simulation runs
numobj = result{1}.numobj;

numtrainO = [];
for jj = 1:length(result)
  numtrainO = [numtrainO, result{jj}.numtrainO];
end

trainhist = cell(length(numtrainO),1);
resultind = 1;
for jj = 1:length(result)
  numresult = length(result{jj}.trainhist);
  trainhist(resultind:resultind+numresult-1) = result{jj}.trainhist;
  resultind = resultind + numresult;
end

% Average performance for all numbers of training examples
detectTrainS  = zeros(numobj,length(numtrainO));
detectTrainNS = zeros(numobj,length(numtrainO));
recogTrainS   = zeros(numobj,length(numtrainO));
recogTrainNS  = zeros(numobj,length(numtrainO));

% Create figures to display results
HfigD = figure('name', 'Detection'); 
HfigR = figure('name', 'Recognition'); 

for kk = 1:length(numtrainO)
  % check whether probabilities calculated for this training set size
  if ~isfield(trainhist{kk},'probS');
    break;
  end

  % extract test groups, and true category labels
  testgroup = trainhist{kk}.testgroup;
  truecat   = imagedata.classes(testgroup);

  % extract test image probabilities for all models
  objllikS   = trainhist{kk}.probS([1:numobj],testgroup,:);
  backllikS  = trainhist{kk}.probS(numobj+1,testgroup,:);
  objllikNS  = trainhist{kk}.probNS([1:numobj],testgroup,:);
  backllikNS = trainhist{kk}.probNS(numobj+1,testgroup,:);

  % detection performance
  [areaS,alarmS,detectS]    = perfDetect(objllikS, backllikS, truecat);
  [areaNS,alarmNS,detectNS] = perfDetect(objllikNS, backllikNS, truecat);
  detectTrainS(:,kk)  = areaS';
  detectTrainNS(:,kk) = areaNS';

  % plot detection
  Hax = subplot(2,1,1,'Parent',HfigD);
  cla(Hax); hold(Hax,'on');
  if plotArea
    plot(Hax, areaNS, areaS, 'ob', 'markersize', 6, 'markerfacecolor', 'b');
    plot(Hax, [0 1], [0 1], '--k', 'linewidth', 2);
    axis(Hax,'square'); axis(Hax, axisArea);
    xlabel(Hax,'Non-Sharing ROC Area');
    ylabel(Hax,'Sharing ROC Area');
    title(Hax,'Scatter plot of ROC Areas');
  else
    for oo = 1:numobj
      plot(Hax, alarmS, detectS(oo,:), '-r', 'linewidth', 2);
      plot(Hax, alarmNS, detectNS(oo,:), '--g', 'linewidth', 2);
    end
    axis(Hax,'square'); axis(Hax, axisROC);
    title(Hax,'ROC curves for all categories');
  end

  Hax = subplot(2,1,2,'Parent',HfigD);
  cla(Hax); hold(Hax,'on');
  plot(Hax, alarmS, mean(detectS), '-r', 'linewidth', 3);
  plot(Hax, alarmNS, mean(detectNS), '--g', 'linewidth', 3);
  axis(Hax,'square'); axis(Hax, axisROC);
  title(Hax,'Mean ROC curves');

  % recognition performance
  [areaS,alarmS,detectS,confS]     = perfRecog(objllikS, truecat);
  [areaNS,alarmNS,detectNS,confNS] = perfRecog(objllikNS, truecat);
  recogTrainS(:,kk)  = areaS';
  recogTrainNS(:,kk) = areaNS';

  % plot recognition
  Hax = subplot(2,2,1,'Parent',HfigR);
  cla(Hax); hold(Hax,'on');
  if plotArea
    plot(Hax, areaNS, areaS, 'ob', 'markersize', 6, 'markerfacecolor', 'b');
    plot(Hax, [0 1], [0 1], '--k', 'linewidth', 2);
    axis(Hax,'square'); axis(Hax, axisArea);
    xlabel(Hax,'Non-Sharing ROC Area');
    ylabel(Hax,'Sharing ROC Area');
    title(Hax,'Scatter plot of ROC Areas');
  else
    for oo = 1:numobj
      plot(Hax, alarmS, detectS(oo,:), '-r', 'linewidth', 2);
      plot(Hax, alarmNS, detectNS(oo,:), '--g', 'linewidth', 2);
    end
    axis(Hax,'square'); axis(Hax, axisROC);
    title(Hax,'ROC curves for all categories');
  end

  Hax = subplot(2,2,3,'Parent',HfigR);
  cla(Hax); hold(Hax,'on');
  plot(Hax, alarmS, mean(detectS), '-r', 'linewidth', 3);
  plot(Hax, alarmNS, mean(detectNS), '--g', 'linewidth', 3);
  axis(Hax,'square'); axis(Hax, axisROC);
  title(Hax,'Mean ROC curves');

  Hax = subplot(2,2,2,'Parent',HfigR);
  imagesc(confS,'Parent',Hax,[0 1]);
  colormap(Hax,gray(256)); axis(Hax,'square');
  title(Hax,'Sharing Confusion');
  Hax = subplot(2,2,4,'Parent',HfigR);
  imagesc(confNS,'Parent',Hax,[0 1]);
  colormap(Hax,gray(256)); axis(Hax,'square');
  title(Hax,'Non-Sharing Confusion');

  input(sprintf('%d training examples...',numtrainO(kk)));
end

% plot performance summary versus training examples
HfigT = figure('name', 'Training Examples'); 
Hax = subplot(2,1,1,'Parent',HfigT);
hold(Hax,'on');
plot(Hax, numtrainO, mean(detectTrainS), '-r', 'linewidth', 3);
plot(Hax, numtrainO, mean(detectTrainNS), '--g', 'linewidth', 3);
Hax = subplot(2,1,2,'Parent',HfigT);
hold(Hax,'on');
plot(Hax, numtrainO, mean(recogTrainS), '-r', 'linewidth', 3);
plot(Hax, numtrainO, mean(recogTrainNS), '--g', 'linewidth', 3);

disp('Detection Sharing:'); disp(mean(detectTrainS));
disp('Detection Non-Sharing:'); disp(mean(detectTrainNS));
disp('Recognition Sharing:'); disp(mean(recogTrainS));
disp('Recognition Non-Sharing:'); disp(mean(recogTrainNS));

