
function [mu, sigma] = mannWhitney(scorePos, scoreNeg)
%mannWhitney    Mann-Whitney tests for significant differences in areas under ROC
%
%    [mu, sigma] = mannWhitney(scorePos, scoreNeg)
%
%  Gaussian asymptotics for area under ROCs of K different tests, where
%    scorePos = MxK matrix of positive example scores (large values good)
%    scoreNeg = NxK matrix of negative example scores (small values good)
%    mu,sigma = estimated mean and covariance of Mann-Whitney rank statistic,
%               which is equivalent to the empirical area under ROC

% determine number of test samples
M = size(scorePos,1);
N = size(scoreNeg,1);
K = size(scorePos,2);
if size(scoreNeg,2) ~= K
  error('Input score matrices inconsistent');
end

% pairwise comparisons between all pairs of scores
diffPos = zeros(M,K);
diffNeg = zeros(N,K);

for kk = 1:K
  allPairs = ((scorePos(:,kk*ones(1,N)) - scoreNeg(:,kk*ones(1,M))') > 0);
  diffPos(:,kk) = mean(allPairs,2);
  diffNeg(:,kk) = mean(allPairs,1)';
end

% estimated mean
mu = mean(diffPos,1)';

% estimated covariance
sigmaPos = (diffPos'*diffPos - M*mu*mu')/(M-1);
sigmaNeg = (diffNeg'*diffNeg - N*mu*mu')/(N-1);
sigma = sigmaPos/M + sigmaNeg/N;

