
function plotGaussMix(mu, sigma, weights, colors, Hax, pointstyle, linestyle)
%plotGaussMix    Plot two-dimensional Gaussian mixtures
%
%    plotGaussMix(mu, sigma, weights, colors, Hax, pointstyle, linestyle)
%
%  Plot two-dimensional mixture of M Gaussians, where
%    mu      = 2xM matrix of mean vectors
%    sigma   = 2x2xM matrix of covariance matrices
%    weights = M-dim. vector of weights (affects intensity of plotted contours)
%    colors  = M-dim. cell array of color characters for each mixture component,
%              or single color for all mixture centers, or 'numbers' to
%              plot mean locations using integer indices rather than points,
%              or 'none' to not plot mean (only covariance ellipse)
%    Hax     = axes on which to create plot (creates new figure by default)
%    pointstyle = cell array of arguments for plotting mixture means
%    linestyle  = cell array of arguments for plotting mixture covariances

  Ncomp = size(mu,2);

  % Default arguments
  plotmeans   = 1;
  plotnumbers = 0;
  if nargin < 4 || isempty(colors)
    colors = cell(1,Ncomp);
    [colors{:}] = deal('b');
  elseif isstr(colors) && isequal(lower(colors),'numbers')
    plotnumbers = 1;
  elseif isstr(colors) && isequal(lower(colors),'none')
    plotmeans = 0;
  elseif ischar(colors)
    ctmp        = colors;
    colors      = cell(1,Ncomp);
    [colors{:}] = deal(ctmp);
  end

  if nargin < 5 || isempty(Hax)
    Hax = axes;
  end
  if nargin < 6 || isempty(pointstyle)
    pointstyle = {'o','MarkerSize',6};
  end
  if nargin < 7 || isempty(linestyle)
    linestyle  = {'-k','LineWidth',2};
  end

  mu = mu(1:2,:);
  %dim = size(mu,1); 
  %if (dim ~= 2)
  %  warning('Gaussian mixture plotting requires dim=2');
  %  return;
  %end

  % Rescale weights to plot intensity
  weights = 0.75*weights/max(weights) + 0.25;

  % Plot covariances
  hold(Hax,'on');
  theta = linspace(0,2*pi,200);
  for (cc = 1:Ncomp)
    [V,D] = eig(sigma(1:2,1:2,cc));

    ellipse = repmat(mu(:,cc),1,length(theta)) + ...
      sqrt(D(1,1))*V(:,1)*sin(theta) + sqrt(D(2,2))*V(:,2)*cos(theta);
    plot(Hax, ellipse(1,:), ellipse(2,:), '-w','LineWidth',3);
    Hplot = plot(Hax, ellipse(1,:), ellipse(2,:), linestyle{:});
    color = get(Hplot, 'Color');
    set(Hplot, 'Color', [1 1 1] - weights(cc)*([1 1 1]-color));
  end

  % Plot means 
  for (cc = 1:Ncomp)
    if plotnumbers
      Hplot = text(mu(1,cc), mu(2,cc), sprintf('%d',cc), 'Parent', Hax);
    elseif plotmeans
      Hplot = plot(Hax, mu(1,cc), mu(2,cc), pointstyle{:});
      set(Hplot, 'Color', colors{cc}, 'MarkerFaceColor', colors{cc});
    end
  end

  axis(Hax,'equal');

